@@ -1,4 +1,4 @@
-/* $Id: animate.cc,v 1.7 2002/02/13 03:53:56 bergo Exp $ */
+/* $Id: animate.cc,v 1.8 2002/06/14 17:32:57 bergo Exp $ */
 
 /*
 
@@ -78,9 +78,12 @@ void FlatAnimatedPiece::destroy() {
 }
 
 void FlatAnimatedPiece::step() {
-  X+=DX;
-  Y+=DY;
+  if (ower > 1) {
+    X+=DX;
+    Y+=DY;
+  }
   if (ower==1) { X=XF; Y=YF; }
   --ower;
+  
 }
 
