@@ -1,4 +1,4 @@
-/* $Id: board.cc,v 1.74 2002/05/02 00:41:03 bergo Exp $ */
+/* $Id: board.cc,v 1.82 2002/06/28 01:33:05 bergo Exp $ */
 
 /*
 
@@ -31,6 +31,7 @@
 #include "chess.h"
 #include "global.h"
 #include "text.h"
+#include "protocol.h"
 
 #include "mainwindow.h"
 
@@ -287,13 +288,6 @@ void Board::setFlipped(bool flip) {
   }
 }
 
-inline bool Board::effectiveFlip() {
-  if (FlipInvert)
-    return(!flipped);
-  else
-    return(flipped);
-}
-
 void Board::supressNextAnimation() {
   ++LockAnimate;
 }
@@ -444,7 +438,7 @@ void Board::renderClock() {
 	for(si=0;si<5;si++) { st[si][0]=st[si][1]=0; }
 	sj=0;
 	k=info[5].length();
-	for(i=0;i<j;i++) {
+	for(i=0;i<k;i++) {
 	  switch(info[5][i]) {
 	  case 'P': st[0][sj]++; break;
 	  case 'R': st[1][sj]++; break;
@@ -456,72 +450,87 @@ void Board::renderClock() {
 	}
 
 	// draw stock pieces
-	if (ef) { we=0; be=48; } else { we=48; be=0; }
+	int eph, my;
+
+	if (global.UseVectorPieces)
+	  eph=40+8;
+	else
+	  eph=cur->side + 8;
+
+	if (ef) { we=0; be=eph; } else { we=eph; be=0; }
+
+	my=2*eph;
+	C.setColor(0x505050);
+	for(si=2;si<my+2;si+=2)
+	  C.drawLine(5,C.Y+si,235,C.Y+si);
+
+
+	for(si=3;si<my+2;si+=2) {
+	  int lv;
+	  lv=my/2 - si;
+	  if (lv<0) lv=-lv;
+	  lv*=2;
+	  if (lv>0xc0) lv=0xc0;
+	  lv=0xc0-lv;
+	  lv=(lv<<16)|(lv<<8)|lv;
+	  C.setColor(lv);
+
+	  C.drawLine(5,C.Y+si,235,C.Y+si);
+	}
+
+	C.setColor(0x555555);
+	for(si=0;si<5;si++)
+	  C.drawLine(40*(si+1),C.Y+2,40*(si+1),C.Y+my+1);
+
 	for(si=0;si<5;si++) {
 	  if (st[si][0]) {
-	    global.vpieces.drawPiece(clkbar,gc,40,40*si,C.Y+we,pk[si]|WHITE);
+
+	    if (global.UseVectorPieces)
+	      global.vpieces.drawPiece(clkbar,gc,40,40*si,C.Y+we,pk[si]|WHITE);
+	    else
+	      cur->drawPiece(pk[si]|WHITE,clkbar,gc,40*si, C.Y+we);
+
 	    sprintf(z,"%d",st[si][0]);
+
+	    shadyText(40*si+20+1, C.Y+eph+we, 2, z);
+
+	    /*
+	    C.setColor(0);
+	    C.drawString(40*si+20+1, C.Y+eph+we, 2, z);
 	    C.setColor(0xffffff);
-	    C.drawString(40*si+20, C.Y+48+we, 2, z);
+	    C.drawString(40*si+20, C.Y+eph+we, 2, z);
+	    */
+
 	    if (mygame->MyColor==WHITE)
-	      addTarget(new DropSource(pk[si]|WHITE,Width()+40*si,C.Y+we,40,48));
+	      addTarget(new DropSource(pk[si]|WHITE,Width()+40*si,C.Y+we,40,eph));
 	  }
 	  if (st[si][1]) {
-	    global.vpieces.drawPiece(clkbar,gc,40,40*si,C.Y+be,pk[si]|BLACK);
+
+	    if (global.UseVectorPieces)
+	      global.vpieces.drawPiece(clkbar,gc,40,40*si,C.Y+be,pk[si]|BLACK);
+	    else
+	      cur->drawPiece(pk[si]|BLACK,clkbar,gc,40*si, C.Y+be);
 
 	    sprintf(z,"%d",st[si][1]);
+
+	    shadyText(40*si+20+1, C.Y+eph+be, 2, z);
+
+	    /*
+	    C.setColor(0);
+	    C.drawString(40*si+20+1,C.Y+eph+be, 2, z);
 	    C.setColor(0xffffff);
-	    C.drawString(40*si+20,C.Y+48+be, 2, z);
+	    C.drawString(40*si+20,C.Y+eph+be, 2, z);
+	    */
+
 	    if (mygame->MyColor==BLACK)
-	      addTarget(new DropSource(pk[si]|BLACK,Width()+40*si,C.Y+be,40,48));
+	      addTarget(new DropSource(pk[si]|BLACK,Width()+40*si,C.Y+be,40,eph));
 	  }
 	} // for
 
-	C.consumeTop(100);
+	C.consumeTop(2*eph+8);
       } // text | graphic stock
     } // if !info[5].empty
 
-    // scratch-board pieces and buttons
-    if (EditMode) {
-      C.consumeTop(10);
-      i = 32;
-
-      C.setColor(0x404040);
-      C.drawRect(10,C.Y,i*6,i*2+8,true);
-      C.setColor(0xffffff);
-      C.drawRect(10,C.Y,i*6,i*2+8,false);
-
-      for(k=0;k<6;k++) {
-	global.vpieces.drawPiece(clkbar,gc, i, 10+i*k, C.Y,WHITE|(k+1));
-	global.vpieces.drawPiece(clkbar,gc, i, 10+i*k, C.Y+i+8,BLACK|(k+1));
-	addTarget(new DropSource(WHITE|(k+1),Width()+10+i*k,C.Y,i,i));
-	addTarget(new DropSource(BLACK|(k+1),Width()+10+i*k,C.Y+i+8,i,i));
-      }
-
-      C.consumeTop(i*2+16);
-
-      C.setColor(0x404040);
-      j=C.stringWidth(2,"Empty");
-      C.drawRect(10,C.Y,j+10,20,true);
-      C.setColor(0xffffff);
-      C.drawRect(10,C.Y,j+10,20,false);
-      C.drawString(15,C.Y+14,2,"Empty");
-
-      addTarget(new DropSource(EMPTY,Width()+10,C.Y,j+10,20,false));
-
-      k=j;
-      C.setColor(0x404040);
-      j=C.stringWidth(2,"Start Position");
-      C.drawRect(40+k,C.Y,j+10,20,true);
-      C.setColor(0xffffff);
-      C.drawRect(40+k,C.Y,j+10,20,false);
-      C.drawString(45+k,C.Y+14,2,"Start Position");
-
-      addTarget(new DropSource(WHITE|BLACK,Width()+40+k,C.Y,j+10,20,false));
-
-      C.consumeTop(20);
-    }
-
     // draw mouse if protocol menu available
     if (global.protocol)
       if (global.protocol->getPlayerActions()) {
@@ -538,11 +547,11 @@ void Board::renderClock() {
       }	
   }
 
-  C.consumeTop(16);
-  j=16;
-
-  if (C.H > 16) {
+  if (C.H > 32) {
     if (position.getAnnotation()) {
+      C.consumeTop(16);
+      j=16;
+
       C.setColor(0x252535);
       C.drawRect(j-5,C.Y,170,C.H,true);
       C.setColor(0x4545ff);
@@ -563,6 +572,20 @@ void Board::setClock(int wsecs,int bsecs,int actv,int cdown) {
   queueRepaint();
 }
 
+void Board::shadyText(int x,int y, int f, char *z) {
+  int i,j;
+  
+  C.setColor(0);
+  for(i=-2;i<=3;i++)
+    for(j=-2;j<=2;j++)
+      C.drawString(x+i,y+j,f,z);
+
+  C.setColor(0x808080);
+  C.drawString(x,y,f,z);
+  C.setColor(0xffffff);
+  C.drawString(x+1,y,f,z);
+}
+
 void Board::freeze() {
   frozen_lvl++;
 }
@@ -635,7 +658,7 @@ void Board::update(bool sndflag=false) {
 	  sok=true; // sound ok
 	  fake=position;
 	  fake.intersection(rpv);
-	  rate = global.SmootherAnimation ? 3 : 2;
+	  rate = global.SmootherAnimation ? 5 : 3;
 	  for(i=0;i<LastMove.size();i++) {
 	    if (ef)
 	      ap=new FlatAnimatedPiece(LastMove[i]->Piece,
@@ -660,10 +683,10 @@ void Board::update(bool sndflag=false) {
 	    gotAnimationLoop=1;
 	    currently_rendered.invalidate();
 	    board_configure_event(yidget,0,(gpointer)this);
-	    if (rate==2)
-	      gtk_timeout_add(global.UseVectorPieces?50:80,board_animate,this);
+	    if (rate==3)
+	      gtk_timeout_add(global.UseVectorPieces?40:70,board_animate,this);
 	    else
-	      gtk_timeout_add(global.UseVectorPieces?30:60,board_animate,this);
+	      gtk_timeout_add(global.UseVectorPieces?30:40,board_animate,this);
 	  }
 	}
   }
@@ -1232,30 +1255,32 @@ gboolean board_expose_event(GtkWidget *widget,GdkEventExpose *ee,
 
   // draw highlighted squares
 
-  if ((me->sp)||(!me->LastMove.empty())||(me->premoveq)) {
+  if (!me->gotAnimationLoop)
+    if ((me->sp)||(!me->LastMove.empty())||(me->premoveq)) {
     
-    if ((global.HilightLastMove)&&(!me->LastMove.empty()))
-      for(i=0;i<me->LastMove.size();i++) {
-	me->outlineRectangle(gc,me->LastMove[i]->SX,me->LastMove[i]->SY,
-			     0x0000ff,3);
-	me->outlineRectangle(gc,me->LastMove[i]->DX,me->LastMove[i]->DY,
-			     0x0000ff,3);
-      }
-    
-    if (me->sp)
-      for(i=0;i<me->sp;i++)
-	me->outlineRectangle(gc,me->ax[i],me->ay[i],me->sel_color,4);
-    
-    if (me->premoveq) {
-      me->outlineRectangle(gc,me->pmx[1],me->pmy[1],0xff0000,3);
-      me->drawBall(gc,me->pmx[1],me->pmy[1],0xff0000,5);
-      if (me->pmx[0]>=0) {
-	me->outlineRectangle(gc,me->pmx[0],me->pmy[0],0xff0000,3);
-	me->drawBall(gc,me->pmx[0],me->pmy[0],0xff0000,5);
+      if (!me->EditMode)
+	if ((global.HilightLastMove)&&(!me->LastMove.empty()))
+	  for(i=0;i<me->LastMove.size();i++) {
+	    me->outlineRectangle(gc,me->LastMove[i]->SX,me->LastMove[i]->SY,
+				 0x0000ff,3);
+	    me->outlineRectangle(gc,me->LastMove[i]->DX,me->LastMove[i]->DY,
+				 0x0000ff,3);
+	}
+      
+      if (me->sp)
+	for(i=0;i<me->sp;i++)
+	  me->outlineRectangle(gc,me->ax[i],me->ay[i],me->sel_color,4);
+      
+      if (me->premoveq) {
+	me->outlineRectangle(gc,me->pmx[1],me->pmy[1],0xff0000,3);
+	me->drawBall(gc,me->pmx[1],me->pmy[1],0xff0000,5);
+	if (me->pmx[0]>=0) {
+	  me->outlineRectangle(gc,me->pmx[0],me->pmy[0],0xff0000,3);
+	  me->drawBall(gc,me->pmx[0],me->pmy[0],0xff0000,5);
+	}
       }
     }
-  }
-
+  
   /* if dragging... */
   if (me->dr_active) {
     me->dr_step=0;
@@ -1284,7 +1309,7 @@ gboolean board_expose_event(GtkWidget *widget,GdkEventExpose *ee,
    [Board::fake], else renders [Board::position].
 
    Copy [scrap] to [buffer]. (X programming gizmo, scrap is an image,
-   position is a pixmap, and bitmap pieces can't be rendered on
+   buffer is a pixmap, and bitmap pieces can't be rendered on
    pixmaps directly)
 
    Coordinates:
@@ -1427,9 +1452,8 @@ static int gcpw=0,gcph=0;
 
    Step 1: Erase previous dragging frame
    Only if not first call on this drag ([dr_step] != 0).
-   Vectorized pieces: copy square from [buffer] to [GCPbuffer],
-   Bitmapped pieces:  copy square from [scrap] to [GCPbuffer].
-   Both: Copy clock from [clkbar] to [GCPbuffer] if needed.
+   copy square from [buffer] to [GCPbuffer],
+   copy clock from [clkbar] to [GCPbuffer] if needed.
    Extruded sets: be sure to erase enough to cover [PieceSet::height]
 
    Step 2: Erase source square
@@ -1437,17 +1461,15 @@ static int gcpw=0,gcph=0;
    [dr_fromstock]).
    Vectorized pieces: draw square and coordinates directly to
     [GCPbuffer].
-   Bitmapped pieces: allocate new image (nsq), draw empty square on
-    it, copy [nsq] to [GCPbuffer].
+   Bitmapped pieces: allocate new image (M[0]), draw empty square on
+    it, copy [M[0]] to [GCPbuffer].
    Extruded Bitmapped pieces: redraw piece on square right above the
     source square.
 
    Step 3: Draw dragged piece
    Vectorized pieces: just draw it on [GCPbuffer]
-   Bitmap pieces: painting the piece is easy, painting the background
-    is hell. Uses tsq, nsq (local) and [scrap] (class) to acquire
-    the correct background, paint piece on [nsq], paint [nsq] on
-    [GCPbuffer].
+   Bitmap pieces: paint alpha'ed piece (more comments to come, experimental
+    code)
 
    Step 3 1/2: Draw coordinates (bitmap-mode only)
    Redraw the coordinates (if global.ShowCoordinates set) on
@@ -1462,13 +1484,12 @@ gboolean board_motion_event(GtkWidget *widget,
 			    GdkEventMotion *em,
 			    gpointer data) {
   Board *me;
-  GdkGC *gc=0, *gcpgc;
-  rgbptr nsq=0, tsq=0, usq=0, vsq=0, ovsq=0;
+  GdkGC *gcpgc;
   int rw,rh;
   bool clock_too=false;
-  bool got_ghost=false;
   int ww,wh,sw,sh;
   int sq,he,xc,xr;
+  int bw,bh;
 
   memcpy(&LastMotionEvent,em,sizeof(GdkEventMotion));
 
@@ -1507,30 +1528,23 @@ gboolean board_motion_event(GtkWidget *widget,
       gdk_draw_pixmap(GCPbuffer,gcpgc,me->buffer,0,0,0,0,sw,sh);
     }
 
+  bw=me->Width();
+  bh=me->Height();
+
   // STEP 1: erase dirty animation square
 
   if (me->dr_step) {
-    rw= me->Width() - me->dr_dirty[0];
-    rh= me->Height() - me->dr_dirty[1];
+    rw= bw - me->dr_dirty[0];
+    rh= bh - me->dr_dirty[1];
     if (rw > sq) { rw = sq; clock_too=true; }
     if (rh > he) { rh = he; clock_too=true; }
  
     if ((rw>0)&&(rh>0))
-      if (global.UseVectorPieces) {
-	gdk_draw_pixmap(GCPbuffer,
-			widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
-			me->buffer,
-			me->dr_dirty[0], me->dr_dirty[1],
-			me->dr_dirty[0], me->dr_dirty[1],
-			rw,rh);
-      } else {
-	gdk_draw_rgb_image(GCPbuffer,widget->style->black_gc,
-			   me->dr_dirty[0], me->dr_dirty[1],
-			   rw, rh,
-			   GDK_RGB_DITHER_NORMAL,
-			   me->scrap+3*(me->dr_dirty[1]*SCRAP_WIDTH+me->dr_dirty[0]),
-			   3 * SCRAP_WIDTH);
-      }
+      gdk_draw_pixmap(GCPbuffer,gcpgc,
+		      me->buffer,
+		      me->dr_dirty[0],me->dr_dirty[1],
+		      me->dr_dirty[0],me->dr_dirty[1],
+		      rw,rh);
   }
 
   if (clock_too) {
@@ -1539,7 +1553,7 @@ gboolean board_motion_event(GtkWidget *widget,
 		    widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
 		    me->clkbar,
 		    0, 0,
-		    me->Width(), 0,250, me->Height() );
+		    bw, 0,250, bh );
     
   }
 
@@ -1548,14 +1562,13 @@ gboolean board_motion_event(GtkWidget *widget,
   if (! me->dr_fromstock) {
     if (global.UseVectorPieces) {
 
-      gc=gdk_gc_new(GCPbuffer);
-      gdk_rgb_gc_set_foreground(gc,((me->dr_c+me->dr_r)&0x01)?global.DarkSqColor:global.LightSqColor);
-      gdk_draw_rectangle(GCPbuffer,gc,TRUE,
+      gdk_rgb_gc_set_foreground(gcpgc,((me->dr_c+me->dr_r)&0x01)?global.DarkSqColor:global.LightSqColor);
+      gdk_draw_rectangle(GCPbuffer,gcpgc,TRUE,
 			 me->borx + me->dr_c*sq, 
 			 me->morey + me->bory + me->dr_r*sq,
 			 sq,sq);
-      gdk_rgb_gc_set_foreground(gc,0);
-      gdk_draw_rectangle(GCPbuffer,gc,FALSE,
+      gdk_rgb_gc_set_foreground(gcpgc,0);
+      gdk_draw_rectangle(GCPbuffer,gcpgc,FALSE,
 			 me->borx + me->dr_c*sq, 
 			 me->morey + me->bory + me->dr_r*sq,
 			 sq,sq);
@@ -1563,30 +1576,31 @@ gboolean board_motion_event(GtkWidget *widget,
       
       // I only need *3, but glibc has some weird glitches when it
       // comes to allocating not padded to 32-bit words
-      nsq=(rgbptr)g_malloc(sq*he*4);
-      me->cur->drawSquare( (me->dr_c+me->dr_r)&0x01, nsq, 0, 0, sq );
+
+      me->M[0].ensure(sq*he*4);
+      me->cur->drawSquare( (me->dr_c+me->dr_r)&0x01, me->M[0].data , 0, 0, sq );
       gdk_draw_rgb_image(GCPbuffer,widget->style->black_gc,
 			 me->borx + me->dr_c * sq, 
 			 me->morey + me->bory + me->dr_r * sq,
 			 sq, sq,
-			 GDK_RGB_DITHER_NORMAL,nsq,3 * sq);
+			 GDK_RGB_DITHER_NORMAL, me->M[0].data ,3 * sq);
 
       // erase upper portion of extruded piece
       if (me->cur->extruded) {
 	if (me->dr_r > 0) { // there's a square above it
-	  vsq=(rgbptr)g_malloc(sq*sq*4);
+	  me->M[1].ensure(sq*sq*4);
 	  xc = me->effectiveFlip() ? 7 - me->dr_c : me->dr_c;
 	  xr = me->effectiveFlip() ? me->dr_r - 1 : 8 - me->dr_r ;
 	  me->cur->drawPieceAndSquare( 
 				      me->position.getPiece(xc, xr) ,
-				      nsq, 0, 0, sq, ((xc+xr)%2 == 0) );
-	  memcpy(vsq,nsq,sq*sq*3);
+				      me->M[0].data, 0, 0, sq, ((xc+xr)%2 == 0) );
+	  memcpy(me->M[1].data , me->M[0].data, sq*sq*3);
 
 	  gdk_draw_rgb_image(GCPbuffer,widget->style->black_gc,
 			     me->borx + me->dr_c * sq, 
 			     me->morey + me->bory + (me->dr_r - 1) * sq,
 			     sq, sq,
-			     GDK_RGB_DITHER_NORMAL,nsq,3 * sq);
+			     GDK_RGB_DITHER_NORMAL,me->M[0].data,3 * sq);
 	} else { // there is nothing above it, just paint it black
 	  gdk_draw_rectangle(GCPbuffer,widget->style->black_gc,TRUE,
 			     me->borx + me->dr_c * sq,
@@ -1624,130 +1638,20 @@ gboolean board_motion_event(GtkWidget *widget,
   if (dy < 0) { y-=dy; dy=0; }
 
   if (global.UseVectorPieces) {
-    if (!gc)
-      gc=gdk_gc_new(GCPbuffer);
     global.vpieces.drawPiece((GdkPixmap *)(GCPbuffer),
-			     gc,sq,dx,dy,me->dr_p);    
+			     gcpgc,sq,dx,dy,me->dr_p);    
     xr=dy;
+    //  } else if (dx+sq > me->Width() ) {
   } else {
-    // this is gonna get tricky...
-    // copy currently rendered bg to nsq...
-
-    if (!nsq)
-      nsq=(rgbptr)g_malloc(sq*he*4);
+    // segment-by-segment masked render, piece overlaps clock area
 
     if (me->cur->extruded) {
       xr=dy-he+sq; xc=he-sq;
       if (xr < 0) { xc+=xr; xr=0; }
       if (xc > me->morey) { --xc; ++xr; }
     } else { xc=0; xr=dy; }
-
-    got_ghost = rects_overlap(dx,xr,sq,sq+xc,me->borx+me->dr_c*sq,
-			      me->morey+me->bory + me->dr_r*sq,sq,sq);
-
-    if (got_ghost) { // fake empty square
-      tsq=(rgbptr)g_malloc(sq*he*4);
-
-      me->cur->bitblt(me->scrap,tsq, 
-		      me->borx+me->dr_c*sq, 
-		      me->morey+me->bory+me->dr_r*sq-xc,SCRAP_WIDTH,
-		      0,0,sq,sq,sq+xc);
-      me->cur->drawSquare( (me->dr_c+me->dr_r)&0x01, me->scrap, 
-			   me->borx+me->dr_c*sq, 
-			   me->morey+me->bory+me->dr_r*sq,
-			   SCRAP_WIDTH );
-
-      // extruded pieces
-      if ((me->cur->extruded)&&(xc>0)) {
-	usq=(rgbptr)g_malloc(sq*sq*4);
-
-	if (me->dr_r == 0) { // no sq above, paint black bg on morey region
-	  memset(nsq,0,sq*sq*3);
-	  me->cur->bitblt(me->scrap,usq,
-			  me->borx+me->dr_c*sq,me->bory,SCRAP_WIDTH,
-			  0,0,sq,sq,me->morey); // save old content
-	  me->cur->bitblt(nsq,me->scrap,
-			  0,0,sq,
-			  me->borx+me->dr_c*sq,me->bory,SCRAP_WIDTH,
-			  sq,me->morey);
-	} else {
-	  // muahahaha
-	  ovsq=(rgbptr) g_malloc(sq*sq*4);
-
-	  me->cur->bitblt(me->scrap, ovsq,
-			  me->borx+me->dr_c*sq,
-			  me->morey+me->bory+(me->dr_r-1)*sq,SCRAP_WIDTH,
-			  0,0,sq,sq,sq);
-
-	  me->cur->bitblt(vsq,me->scrap,
-			  0,0,sq,
-			  me->borx+me->dr_c*sq,
-			  me->morey+me->bory+(me->dr_r-1)*sq,SCRAP_WIDTH,
-			  sq,sq);
-	}
-
-      } // if extruded and xc > 0
-
-    } // if got_ghost
-
-    me->cur->bitblt(me->scrap,nsq,
-		    dx, dy, SCRAP_WIDTH,
-		    0,0,sq,sq,sq);
-
-    // draw alpha'ed piece on nsq
-    me->cur->drawPiece(me->dr_p,nsq,0,0,sq);
-
-    // draw nsq on board
-    gdk_draw_rgb_image(GCPbuffer,widget->style->black_gc,
-		       dx, dy,
-		       sq, sq,
-		       GDK_RGB_DITHER_NORMAL,nsq,3 * sq);
-
-    // draw upper portion (extruded only)
-    if ((me->cur->extruded)&&(xc>0)) {
-
-      // background from scrap
-      me->cur->bitblt(me->scrap,nsq,dx,xr,SCRAP_WIDTH,
-		      0,0,sq,sq,xc);
-      
-      // draw piece
-      me->cur->drawPiece(me->dr_p,nsq,0,xc,sq);
-      
-      // paste everything (piece+background) on the destination buffer
-      gdk_draw_rgb_image(GCPbuffer,widget->style->black_gc,
-			 dx, xr,
-			 sq, xc,
-			 GDK_RGB_DITHER_NORMAL,nsq,3 * sq);
-      
-    } // if extruded and xc > 0
-
-    // undo ghost effect
-    if (got_ghost) { // undo the trick
-      me->cur->bitblt(tsq,me->scrap, 0,0, sq,
-		      me->borx+me->dr_c*sq,
-		      me->morey+me->bory+me->dr_r*sq-xc,
-		      SCRAP_WIDTH, sq, sq+xc);
-
-      if ((me->dr_r == 0)&&(xc>0))
-	me->cur->bitblt(usq,me->scrap, 0,0, sq,
-			me->borx+me->dr_c*sq,me->bory,SCRAP_WIDTH,
-			sq, me->morey);
-
-      if (ovsq) {
-	  me->cur->bitblt(ovsq,me->scrap,
-			  0,0,sq,
-			  me->borx+me->dr_c*sq,
-			  me->morey+me->bory+(me->dr_r-1)*sq,SCRAP_WIDTH,
-			  sq,sq);
-	  g_free(ovsq);
-	  g_free(vsq);
-      }
-
-      if (usq) g_free(usq);
-      g_free(tsq);
-    }
-
-    g_free(nsq);
+    
+    me->cur->drawPiece(me->dr_p,GCPbuffer,gcpgc,dx,dy);
   }
 
   // STEP 3 1/2: draw coordinates if not in vector mode
@@ -1762,8 +1666,6 @@ gboolean board_motion_event(GtkWidget *widget,
   me->dr_dirty[1]=xr;
   me->dr_step++;
   
-  if (gc)
-    gdk_gc_destroy(gc);
   return TRUE;
 }
 
@@ -1876,11 +1778,17 @@ gboolean board_button_press_event(GtkWidget *widget,GdkEventButton *be,
 	  gtk_timeout_add(90,board_drag_start_timeout,
 			  (gpointer)(&(me->dr_fto)));
 	} else {
+	  EditBoard *eme;
+	  eme=(EditBoard *)data;
+
 	  // empty / start pos buttons
 	  if ((me->EditMode)&&(me->mygame)) {
 	    switch(ds->P) {
 	    case EMPTY: me->mygame->editEmpty(); break;
 	    case WHITE|BLACK: me->mygame->editStartPos(); break;
+	    case WHITE: eme->popRunEngine(ds->X,ds->Y2); break;
+	    case BLACK: eme->flipSide(); break;
+	    case WASPAWN: eme->getFEN(); break;
 	    }
 	  }
 	}
@@ -2007,10 +1915,8 @@ gboolean vec_board_animate(gpointer data) {
 gboolean board_animate(gpointer data) {
   list<AnimatedPiece *>::iterator li;
   Board *me;
-  rgbptr scrap_copy;
-  int offset;
-  int dc=0, da=0;
-  int minx,miny,maxx,maxy,x,y,nonover=0;
+  Rect *r;
+  int dy,nonover=0;
   int sh0=0,sh1=0;
 
   global.debug("Board","board_animate");
@@ -2021,15 +1927,7 @@ gboolean board_animate(gpointer data) {
   
   me=(Board *)data;
 
-  if (!me->animes.empty()) {
-    scrap_copy=(rgbptr)g_malloc(SCRAP_WIDTH*SCRAP_WIDTH*3);
-    memcpy(scrap_copy,me->scrap,SCRAP_WIDTH*SCRAP_WIDTH*3);
-    da=1;
-    minx=7000;
-    miny=7000;
-    maxx=0;
-    maxy=0;
-  }
+  dy=me->cur->side - me->cur->height;
 
   for(li=me->animes.begin();li!=me->animes.end();li++) {
     if (! (*li)->over() )
@@ -2038,29 +1936,38 @@ gboolean board_animate(gpointer data) {
       sh0++;
   }
 
+  // clear previous frames
+
+  while(! me->adirty.empty() ) {
+    r=me->adirty.front();
+
+    gdk_draw_pixmap(me->yidget->window,
+		    me->yidget->style->black_gc,
+		    me->buffer,
+		    r->x, r->y,
+		    r->x, r->y,
+		    r->w, r->h);
+
+    delete r;
+    me->adirty.pop_front();
+  }
+
+  // draw current frames
+
   for(li=me->animes.begin();li!=me->animes.end();li++)
     if (nonover) {
-      // to erase previous frame
-      x=(*li)->getX();
-      y=(*li)->getY();
-      if (x<minx) minx=x;
-      if (y<miny) miny=y;
-      x+=me->sqside;
-      y+=me->sqside;
-      if (x>maxx) maxx=x;
-      if (y>maxy) maxy=y;
 
       (*li)->step();
 
-      me->cur->drawPiece((*li)->getPiece(),scrap_copy,
-			 x=(*li)->getX(),y=(*li)->getY(),SCRAP_WIDTH);
-      dc++;
-      if (x<minx) minx=x;
-      if (y<miny) miny=y;
-      x+=me->sqside;
-      y+=me->sqside;
-      if (x>maxx) maxx=x;
-      if (y>maxy) maxy=y;
+      r=new Rect((*li)->getX(), (*li)->getY(),
+		 me->cur->side, me->cur->height);
+
+      me->cur->drawPiece((*li)->getPiece(),
+			 me->yidget->window,
+			 me->yidget->style->black_gc,
+			 r->x, r->y);
+      r->translate(0,dy);
+      me->adirty.push_back(r);
     } 
 
   // count remaining
@@ -2083,16 +1990,6 @@ gboolean board_animate(gpointer data) {
   if ((sh0)&&(!sh1))
     global.flushSound();
   
-  if (dc) {
-    offset=3*(miny*SCRAP_WIDTH+minx);
-    gdk_draw_rgb_image(me->yidget->window,me->yidget->style->black_gc,
-		       minx,miny,maxx-minx+1,maxy-miny+1,
-		       GDK_RGB_DITHER_NORMAL,scrap_copy+offset,SCRAP_WIDTH*3);
-  }
-
-  if (da)
-    g_free(scrap_copy);
-  
   if (not_over)
     return TRUE;
   else {
@@ -2146,3 +2043,258 @@ EditBoard::EditBoard(ChessGame *cg) : Board() {
   setGame(cg);
 }
 
+static int popx=0, popy=0;
+
+void poprun_cb(GtkMenu *m,gint *x,gint *y,gpointer data) {
+  *x=popx;
+  *y=popy;
+}
+
+static EditBoard *eb_last=0;
+
+void eb_runengine_nobm(GtkMenuItem *item,gpointer data) {
+  int *i;
+  EngineProtocol *ep;
+  Position p;
+
+  if (!eb_last) return;
+  if (!eb_last->mygame) return;
+  
+  i=(int *)data;
+  
+  switch(*i) {
+  case 0: ep=new GnuChess4Protocol(); break;
+  case 1: ep=new CraftyProtocol();    break;
+  case 2: ep=new SjengProtocol();     break;
+  case 3: ep=new XBoardProtocol();    break;
+  default: return;
+  }
+
+  p=eb_last->mygame->getCurrentPosition();
+  ep->setInitialPosition(&p);
+  global.chandler->openEngine(ep,0);
+}
+
+void eb_runengine_bm(GtkMenuItem *item,gpointer data) {
+  EngineBookmark *ebm;
+  EngineProtocol *ep;
+  Position p;
+
+  if (!eb_last) return;
+  if (!eb_last->mygame) return;
+
+  ebm=(EngineBookmark *)data;
+
+  if (!ebm) return;
+
+  switch(ebm->proto) {
+  case 0: ep=new XBoardProtocol();    break;
+  case 1: ep=new CraftyProtocol();    break;
+  case 2: ep=new SjengProtocol();     break;
+  case 3: ep=new GnuChess4Protocol(); break;
+  default: return;
+  }
+
+  p=eb_last->mygame->getCurrentPosition();
+  ep->setInitialPosition(&p);
+  global.chandler->openEngine(ep,ebm);
+}
+
+void EditBoard::popRunEngine(int x,int y) {
+  GtkWidget *popmenu, *note=0, *e;
+  vector<GtkWidget *> mi;  
+  list<EngineBookmark *>::iterator ei;
+  int i;
+  static int sindex[4]={0,1,2,3};
+
+  eb_last=this;
+
+  popmenu=gtk_menu_new();
+
+  mi.push_back(gtk_menu_item_new_with_label("GNU Chess 4..."));
+  mi.push_back(gtk_menu_item_new_with_label("Crafty..."));
+  mi.push_back(gtk_menu_item_new_with_label("Sjeng..."));
+  mi.push_back(gtk_menu_item_new_with_label("Generic XBoard Engine..."));
+  mi.push_back(gtk_menu_item_new());
+
+  for(i=0;i<4;i++)
+    gtk_signal_connect(GTK_OBJECT(mi[i]),"activate",
+		       GTK_SIGNAL_FUNC(eb_runengine_nobm),
+		       (gpointer) (&sindex[i]) );
+  
+  if (!global.EnginePresets.empty()) {
+    note=gtk_menu_item_new_with_label("If you pick a bookmark, the engine\n"\
+				      "will play the next move, ignoring\n"\
+				      "the side setting in the bookmark.");
+    gtk_widget_set_sensitive(note,FALSE);
+  }
+
+  for(ei=global.EnginePresets.begin();ei!=global.EnginePresets.end();ei++) {
+    e=gtk_menu_item_new_with_label((*ei)->caption.c_str());
+    mi.push_back(e);
+
+    gtk_signal_connect(GTK_OBJECT(e),"activate",
+		       GTK_SIGNAL_FUNC(eb_runengine_bm),
+		       (gpointer)(*ei));
+  }
+
+  if (note)
+    mi.push_back(gtk_menu_item_new());
+
+  for(i=0;i<mi.size();i++) {
+    gtk_widget_show(mi[i]);
+    gtk_menu_append(GTK_MENU(popmenu),mi[i]);
+  }
+
+  if (note) {
+    gtk_widget_show(note);
+    gtk_menu_append(GTK_MENU(popmenu), note);
+  }
+
+  gdk_window_get_origin(widget->window,&popx,&popy);
+
+  popx+=x;
+  popy+=y;
+
+  gtk_menu_popup(GTK_MENU(popmenu),0,0,
+		 (GtkMenuPositionFunc) poprun_cb,
+		 0,1,time(0));
+}
+
+void EditBoard::renderClock() {
+  GdkGC *gc=clkgc;
+  int lch=clock_ch;
+  int i,j,k;  
+
+  Board::renderClock();
+  if (!lch) return;
+  if (!mygame) return;
+
+  // scratch-board pieces and buttons
+  C.consumeTop(10);
+  i = 32;
+
+  for(k=0;k<6;k++) {
+    C.drawButton(10+i*k, C.Y, i, i, NULL, 0, 0xffffff, 0x505070);
+    C.drawButton(10+i*k, C.Y+i+8, i, i, NULL, 0, 0xffffff, 0x505070);
+    
+    global.vpieces.drawPiece(clkbar,gc, i, 10+i*k, C.Y,WHITE|(k+1));
+    global.vpieces.drawPiece(clkbar,gc, i, 10+i*k, C.Y+i+8,BLACK|(k+1));
+    addTarget(new DropSource(WHITE|(k+1),Width()+10+i*k,C.Y,i,i));
+    addTarget(new DropSource(BLACK|(k+1),Width()+10+i*k,C.Y+i+8,i,i));
+  }
+  
+  C.consumeTop(i*2+16);
+  
+  j=C.drawButton(10,C.Y,-1,20,"Empty",2,0xffffff, 0x505070);
+  addTarget(new DropSource(EMPTY,Width()+10,C.Y,j,20,false));
+  
+  k=10+j+10;
+  j=C.drawButton(k,C.Y,-1,20,"Initial Position",2,0xffffff,0x505070);
+  
+  addTarget(new DropSource(WHITE|BLACK,Width()+k,C.Y,j,20,false));
+
+  k+=10+j;
+  j=C.drawButton(k,C.Y,-1,20,"From FEN",2,0xffffff,0x505070);
+
+  addTarget(new DropSource(WASPAWN,Width()+k,C.Y,j,20,false));
+  
+  C.consumeTop(28);
+  
+  j=C.drawButton(10,C.Y,-1,20,"Run Engine...",
+		 2,0xffffff,0xa07050);
+  addTarget(new DropSource(WHITE,Width()+10,C.Y,j,20,false));
+
+  k=j+20;
+  j+=20+C.stringWidth(2,"Side to move: ");
+
+  C.drawButton( k,C.Y,j-k+10+24,20,NULL,2,0xffffff,0x505070);
+  addTarget(new DropSource(BLACK,Width()+k,C.Y,j-k+10+24,20,false));
+
+  C.drawString( k+10, C.Y+14, 2, "Side to move: ");
+
+  C.setColor( mygame->getSideHint() ? 0xffffff : 0);
+  C.drawEllipse( 10+j+3, C.Y+3, 14, 14, true);
+  C.setColor(0xffffff);
+  C.drawEllipse( 10+j+3, C.Y+3, 14, 14, false);  
+
+  C.consumeTop(20);
+}
+
+void EditBoard::flipSide() {
+  if (mygame) {
+    mygame->setSideHint( ! mygame->getSideHint() );
+    clock_ch=1;
+    queueRepaint();
+  }
+}
+
+void EditBoard::getFEN() {
+  (new GetFENDialog(this))->show();
+}
+
+void EditBoard::applyFEN(string &s) {
+  Position p;
+
+  p.setFEN(s.c_str());
+
+  mygame->editEmpty();
+  mygame->updatePosition(p,1,p.sidehint ? 0 : 1,
+			 0,0,"<FEN>",false);  
+}
+
+GetFENDialog::GetFENDialog(EditBoard *_owner) 
+  : ModalDialog("Enter FEN Position")
+{
+  GtkWidget *v,*hs,*hb,*ok,*cancel;
+
+  owner=_owner;
+  
+  gtk_window_set_default_size(GTK_WINDOW(widget), 450, 100);
+  
+  v=gtk_vbox_new(FALSE,4);
+
+  gtk_container_add(GTK_CONTAINER(widget),v);
+
+  e=gtk_entry_new();
+  gtk_box_pack_start(GTK_BOX(v),e,FALSE,TRUE,4);
+
+  hs=gtk_hseparator_new();
+  gtk_box_pack_start(GTK_BOX(v),hs,FALSE,TRUE,4);
+
+  hb=gtk_hbutton_box_new();
+  gtk_button_box_set_layout(GTK_BUTTON_BOX(hb), GTK_BUTTONBOX_END);
+  gtk_button_box_set_spacing(GTK_BUTTON_BOX(hb), 5);
+  gtk_box_pack_start(GTK_BOX(v), hb, FALSE, TRUE, 2); 
+
+  ok=gtk_button_new_with_label("Ok");
+  GTK_WIDGET_SET_FLAGS(ok,GTK_CAN_DEFAULT);
+  cancel=gtk_button_new_with_label("Cancel");
+  GTK_WIDGET_SET_FLAGS(cancel,GTK_CAN_DEFAULT);
+  gtk_box_pack_start(GTK_BOX(hb),ok,TRUE,TRUE,0);
+  gtk_box_pack_start(GTK_BOX(hb),cancel,TRUE,TRUE,0);
+  gtk_widget_grab_default(ok);
+
+  gtk_widget_show(ok);
+  gtk_widget_show(cancel);
+  gtk_widget_show(hb);
+  gtk_widget_show(hs);
+  gtk_widget_show(e);
+  gtk_widget_show(v);
+
+  setDismiss(GTK_OBJECT(cancel),"clicked");
+
+  gtk_signal_connect(GTK_OBJECT(ok),"clicked",
+		     GTK_SIGNAL_FUNC(getfen_ok),(gpointer)this);
+
+  focused_widget=e;
+}
+
+void getfen_ok(GtkWidget *w, gpointer data) {
+  string s;
+  GetFENDialog *me;
+  me=(GetFENDialog *)data;
+  s=gtk_entry_get_text(GTK_ENTRY(me->e));
+  me->owner->applyFEN(s);
+  me->release();
+}
