@@ -1,4 +1,4 @@
-/* $Id: board.h,v 1.40 2002/04/23 07:55:09 bergo Exp $ */
+/* $Id: board.h,v 1.44 2002/06/28 01:33:05 bergo Exp $ */
 
 /*
 
@@ -41,6 +41,14 @@
 // foreign classes referenced here
 class ChessGame;
 
+class Rect {
+ public:
+  Rect() { x=0; y=0; w=0; h=0; }
+  Rect(int a,int b,int c,int d) { x=a; y=b; w=c; h=d; }
+  void translate(int dx,int dy) { x+=dx; y+=dy; }
+  int x,y,w,h;
+};
+
 class DropSource {
  public:
   DropSource(piece p, int x1,int y1,int w,int h, bool d=true);
@@ -66,6 +74,29 @@ class RootBoard {
   void clockString(int val,char *z);
 };
 
+// I just can't work out the correct syntax to put the
+// bodies of methods in the .cc file, if you know, email me
+template<class T> 
+class mblock {
+ public:
+
+  mblock(int _sz=0) { data=0; sz=0; if (_sz) ensure(_sz); }
+
+  ~mblock() { if (sz) g_free((gpointer) data); sz=0; data=0; }
+
+  bool ensure(int _sz) {
+    if (_sz <= sz) return true; // never shrink
+    if (sz==0) data=(T *) g_malloc(_sz * sizeof(T));
+    else data=(T *) g_realloc((gpointer) data, _sz * sizeof(T));
+    sz=_sz;
+    if (!data) cerr << "eboard ** mblock error, expect oddities\n";
+    return(data!=0);
+  }
+  
+  T * data;
+  int sz;
+};
+
 class Board : public WidgetProxy,
 	      public ClockHost,
 	      public RootBoard,
@@ -116,7 +147,7 @@ class Board : public WidgetProxy,
   void openMovelist();
   bool hasGame();
 
-  bool effectiveFlip();
+  bool effectiveFlip() { return(FlipInvert?(!flipped):flipped); }
 
   void dump();
 
@@ -127,15 +158,31 @@ class Board : public WidgetProxy,
  
   static Board * PopupOwner;
 
+  virtual void renderClock();
+
+ protected:
+  GdkPixmap *clkbar;
+  GdkGC *clkgc;
+  LayoutBox C;
+  int clock_ch;
+
+  ChessGame *mygame;
+
+  // inline calculations
+  int Width()  { return( (sqside<<3) + (borx<<1) ); }
+  int Height() { return( (sqside<<3) + (bory<<1) + morey); }
+
  private:
   GtkWidget *yidget;
 
   PieceSet *cur;
-  GdkPixmap *buffer,*clkbar;
-  GdkGC *clkgc, *wgc;
+  GdkPixmap *buffer;
+  GdkGC *wgc;
   GdkFont *f1,*f2,*f3;
   rgbptr scrap;
 
+  mblock<unsigned char> M[2];
+
   bool canselect;
   bool allowMove;
   bool FlipInvert;
@@ -156,11 +203,9 @@ class Board : public WidgetProxy,
   Position currently_rendered;
   Position fake;
 
-  ChessGame *mygame;
   ChessClock clock;
-  LayoutBox C;
 
-  int hilite_ch, clock_ch;
+  int hilite_ch;
 
   // selection/highlight
   int ax[2],ay[2],sp;
@@ -179,6 +224,9 @@ class Board : public WidgetProxy,
   int   dr_dirty[2];
   bool  dr_fto;
 
+  // animation
+  list<Rect *> adirty;
+
   // info lines
   string info[6];
 
@@ -190,13 +238,8 @@ class Board : public WidgetProxy,
 
   stack<bool> UpdateStack;
 
-  // inline calculations
-  int Width()  { return( (sqside<<3) + (borx<<1) ); }
-  int Height() { return( (sqside<<3) + (bory<<1) + morey); }
-
   void popupDropMenu(int col,int row,int sx,int sy,guint32 etime);
   void popupProtocolMenu(int x,int y, guint32 etime);
-  void renderClock();
   void sendMove();
   void outlineRectangle(GdkGC *gc, int x,int y,int color,int pen);
   void drawBall(GdkGC *gc, int x,int y,int color,int radius);
@@ -206,6 +249,8 @@ class Board : public WidgetProxy,
   void composeVecBoard(GdkGC *gc);
   void pasteVecBoard();
 
+  void shadyText(int x,int y, int f, char *z);
+
   friend gboolean board_animate(gpointer data);
   friend gboolean vec_board_animate(gpointer data);
 
@@ -239,6 +284,27 @@ class Board : public WidgetProxy,
 class EditBoard : public Board {
  public:
   EditBoard(ChessGame *cg);
+  void popRunEngine(int x,int y);
+  void flipSide();
+  void getFEN();
+  void applyFEN(string &s);
+
+  virtual void renderClock();
+
+ private:
+  friend void eb_runengine_bm(GtkMenuItem *item,gpointer data);
+  friend void eb_runengine_nobm(GtkMenuItem *item,gpointer data);
+};
+
+class GetFENDialog : public ModalDialog {
+ public:
+  GetFENDialog(EditBoard *_owner);
+
+ private:
+  EditBoard *owner;
+  GtkWidget *e;
+
+  friend void getfen_ok(GtkWidget *w, gpointer data);
 };
 
 #endif
