@@ -1,4 +1,4 @@
-/* $Id: chess.cc,v 1.60 2002/05/02 00:41:04 bergo Exp $ */
+/* $Id: chess.cc,v 1.62 2002/06/22 18:42:42 bergo Exp $ */
 
 /*
 
@@ -424,10 +424,16 @@ void ChessGame::editEmpty() {
   last_half_move=1;
 
   for(i=0;i<8;i++) for(j=0;j<8;j++) p.setPiece(i,j,EMPTY);
+  p.sidehint=true;
+
   moves.push_back(p);
   cursor=moves.end();
   cursor--;
-  if (myboard) { myboard->setInfo(2,(*cursor).getLastMove()); myboard->update(); }
+  if (myboard) { 
+    myboard->setInfo(2,(*cursor).getLastMove()); 
+    myboard->setInfo(4,p.getMaterialString());
+    myboard->update();
+  }
 }
 
 void ChessGame::editStartPos() {
@@ -436,12 +442,17 @@ void ChessGame::editStartPos() {
   if (!LocalEdit) return;
 
   moves.clear();
+  startpos.setStartPos(); // sidehint might need to be fixed
   last_half_move=1;
 
   moves.push_back(p);
   cursor=moves.end();
   cursor--;
-  if (myboard) { myboard->setInfo(2,(*cursor).getLastMove()); myboard->update(); }
+  if (myboard) {
+    myboard->setInfo(2,(*cursor).getLastMove());
+    myboard->setInfo(4,p.getMaterialString());
+    myboard->update();
+  }
 }
 
 void ChessGame::sendDrop(piece p, int x, int y) {
@@ -523,6 +534,14 @@ void ChessGame::sendMove(int x1,int y1,int x2,int y2) {
 
 }
 
+bool ChessGame::getSideHint() {
+  return (getCurrentPosition().sidehint);
+}
+
+void ChessGame::setSideHint(bool white) {
+  getCurrentPosition().sidehint = white;
+}
+
 void ChessGame::flipHint(int flip) {
   if (myboard)
     myboard->setFlipped(flip!=0);
@@ -812,7 +831,7 @@ bool ChessGame::ParsePgnGame(istream &f,
 			     ChessGame *updatee = NULL) {
   /* 
      ok, now (0.4.x) we do this in a DFA-like manner
-     0  = state 0, initial state (whitespace before movetext)
+     0  = state 0 (whitespace before movetext)
 
      PGN header states:
      1  = found [, consuming PGN field name
@@ -824,9 +843,12 @@ bool ChessGame::ParsePgnGame(istream &f,
      7  = inside { comment
      8 = like level 0, but can't have headers
      9 = got result, waiting end of line (== end of game)
+
+     10 = initial state ;-)
+          whitespace (and comments like in twic pgns) between games
   */
 
-  int   state    = 0;
+  int   state    = 10;
   int   halfmove = 0;
   piece color    = WHITE;
 
@@ -873,7 +895,7 @@ bool ChessGame::ParsePgnGame(istream &f,
 
   omega[0]=0;
 
-  // #define PGN_DEBUG 1
+  //  #define PGN_DEBUG 1
 
 #ifdef PGN_DEBUG
   cerr << "entering new game\n";
@@ -922,6 +944,9 @@ bool ChessGame::ParsePgnGame(istream &f,
 
       switch(state) {
 	// whitespace
+      case 10:
+	if (*p == '[') { state=1; memset(alpha,0,256); ap=0; }
+	break;
       case 0:
 	if (*p == '[')         { state=1; memset(alpha,0,256); ap=0; break; }
       case 8:
