@@ -1,4 +1,4 @@
-/* $Id: chess.h,v 1.37 2002/04/28 21:04:34 bergo Exp $ */
+/* $Id: chess.h,v 1.38 2002/06/14 17:32:57 bergo Exp $ */
 
 /*
 
@@ -133,6 +133,9 @@ class ChessGame : public MoveListListener {
   void editEmpty();
   void editStartPos();
 
+  bool getSideHint(); // true=white to move, used in scratch boards
+  void setSideHint(bool white);
+
   static const char *variantName(variant v);
   static variant variantFromName(char *p);
 
