@@ -1,4 +1,4 @@
-/* $Id: dlg_connect.h,v 1.2 2001/04/28 02:58:57 bergo Exp $ */
+/* $Id: dlg_connect.h,v 1.3 2002/06/02 21:17:13 bergo Exp $ */
 
 /*
 
@@ -50,4 +50,31 @@ class ConnectDialog : public WidgetProxy {
 			     gpointer data);
 };
 
+class EditEngineBookmarksDialog : public ModalDialog {
+ public:
+  EditEngineBookmarksDialog(BookmarkListener *updatee);
+  virtual ~EditEngineBookmarksDialog();
+ private:
+  BookmarkListener *beholder;
+  GtkWidget *bl;
+  GtkWidget *fe[7], *rm, *rmall, *apply;
+  BoxedLabel *uf[2];
+  int selindex;
+
+  void refresh();
+  void updateRightPane();
+
+  friend void eebmd_rowsel(GtkCList *clist,int row,int column,GdkEventButton *eb,
+			   gpointer data);
+  friend void eebmd_rowunsel(GtkCList *clist,int row,int column,GdkEventButton *eb,
+			     gpointer data);
+
+  friend void eebmd_rm1(GtkWidget *w, gpointer data);
+  friend void eebmd_rmall(GtkWidget *w, gpointer data);
+  friend void eebmd_apply(GtkWidget *w, gpointer data);
+
+};
+
+typedef EditEngineBookmarksDialog * EEBMDp;
+
 #endif
