@@ -1,4 +1,4 @@
-/* $Id: dlg_gamelist.cc,v 1.18 2002/03/29 05:23:37 bergo Exp $ */
+/* $Id: dlg_gamelist.cc,v 1.20 2002/04/24 01:16:33 bergo Exp $ */
 
 /*
 
@@ -391,7 +391,8 @@ void StockListDialog::refresh() {
   char *zp[7];
   int i,j;
 
-  char ka[256], kb[256], *kp;
+  char ka[256], kb[256];
+  string *kp;
   int kids[4];
 
   vector<GtkCTreeNode *> filenodes;
@@ -399,7 +400,7 @@ void StockListDialog::refresh() {
   vector<int>    count;
 
   GtkCTreeNode *myfile;
-  TString T(512);
+  tstring t;
 
   static char *e="e";
   static char *es="es";
@@ -484,13 +485,16 @@ void StockListDialog::refresh() {
   }
 
   for(i=0;i<count.size();i++) {
-    strncpy(ka, filenames[i].c_str(), 256);
-    ka[255]=0;
-    T.set( ka );
+    memset(ka,0,256);
+    filenames[i].copy(ka,255);
+    t.set( filenames[i] );
 
     do {
-      kp=T.token("/\n\r");
-      if (kp) { strncpy(ka, kp, 256); ka[255]=0; }
+      kp=t.token("/\n\r");
+      if (kp) { 
+	memset(ka,0,256);
+	kp->copy(ka,255);
+      }
     } while(kp);
 
     snprintf(kb,256,"%s (%d gam%s)", ka, count[i], count[i]==1?e:es);
@@ -540,8 +544,8 @@ void StockListDialog::open() {
   b->reset();
   cg->setBoard(b);
   b->setGame(cg);
-  b->setCanMove(0);
-  b->setSensitive(0);
+  b->setCanMove(false);
+  b->setSensitive(false);
   b->walkBackAll();  
   cg->acknowledgeInfo();
   
