@@ -1,4 +1,4 @@
-/* $Id: eboard.h,v 1.26 2002/04/23 07:55:09 bergo Exp $ */
+/* $Id: eboard.h,v 1.28 2002/06/26 01:06:59 bergo Exp $ */
 
 /*
 
@@ -55,7 +55,9 @@ typedef short int piece;
 typedef unsigned char * rgbptr;
 typedef unsigned char hati;
 
-class ProtocolParser;
+class Protocol;
+class EngineProtocol;
+class EngineBookmark;
 
 class PieceProvider {
  public:
@@ -67,12 +69,14 @@ class InputHandler {
   virtual void setPasswordMode(int pm)=0;
   virtual void userInput(char *text)=0;
   virtual void peekKeys(GtkWidget *who)=0;
+  virtual void updatePrefix()=0;
 };
 
 class ConnectionHandler {
  public:
   virtual void openServer(char *host,int port,
-			  ProtocolParser *protocol,char *helper)=0;
+			  Protocol *protocol,char *helper)=0;
+  virtual void openEngine(EngineProtocol *protocol, EngineBookmark *bm)=0;
 };
 
 class UpdateInterface {
