@@ -2391,7 +2391,7 @@ gtk_ftext_get_last_line(GtkFText * ftext)
 
 
 gint
-gtk_ftext_find_text_upwards(GtkFText * ftext, char *needle,
+gtk_ftext_find_text_upwards(GtkFText * ftext, const char *needle,
 			    int topline, int bottomline,
 			    gboolean hilight)
 {
