@@ -1,4 +1,4 @@
-/* $Id: global.cc,v 1.63 2002/05/02 00:41:04 bergo Exp $ */
+/* $Id: global.cc,v 1.66 2002/06/26 01:06:59 bergo Exp $ */
 
 /*
 
@@ -40,7 +40,11 @@
 
 #include "global.h"
 #include "text.h"
+
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+
 #include "chess.h"
 #include "tstring.h"
 #include "notebook.h"
@@ -111,6 +115,7 @@ Global::Global() {
   mainpaned=0;
   bugpane=0;
   pieceset=0;
+  toplevelwidget=0;
 
   LastScratch = 0;
 
@@ -1357,3 +1362,5 @@ Environment::Environment() {
   }
 }
 
+
+
