@@ -1,4 +1,4 @@
-/* $Id: global.h,v 1.59 2002/04/27 14:43:43 bergo Exp $ */
+/* $Id: global.h,v 1.61 2002/06/26 01:06:59 bergo Exp $ */
 
 /*
 
@@ -44,7 +44,7 @@ class OutputPane;
 class TextSet;
 class NetConnection;
 class Status;
-class ProtocolParser;
+class Protocol;
 class ChessGame;
 class Notebook;
 class QuickBar;
@@ -284,7 +284,7 @@ class Global : public ChannelSplitter,
   OutputPane         *output;
   NetConnection      *network;
   Status             *status;
-  ProtocolParser     *protocol;
+  Protocol           *protocol;
   ConnectionHandler  *chandler;
   PieceProvider      *promotion;
   Notebook           *ebook;
@@ -296,6 +296,7 @@ class Global : public ChannelSplitter,
   GtkWidget          *killbox;
   GtkWidget          *lowernotebook;
   GtkWidget          *mainpaned;
+  GtkWidget          *toplevelwidget;
   BugPane            *bugpane;
 
   StringCollection   annotator;
@@ -374,6 +375,8 @@ class Global : public ChannelSplitter,
 
   Environment env;
 
+  string ConsoleReply;
+
  private:
   int createDir(char *z);
   void renumberGame(ChessGame *cg,int id);
