@@ -1,4 +1,4 @@
-/* $Id: help.cc,v 1.9 2002/04/24 01:16:33 bergo Exp $ */
+/* $Id: help.cc,v 1.12 2002/06/26 01:06:59 bergo Exp $ */
 
 /*
 
@@ -23,9 +23,18 @@
 
 */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
@@ -62,7 +71,8 @@ namespace Help {
 	   "Ctrl+G: Find Previous\n"\
 	   "\nInput box:\n"\
 	   "Up/Down (arrows): move thru input history\n"\
-	   "Enter: send text line");
+	   "Enter: send text line\n"\
+	   "Esc: switch Chat/Command Mode");
     
     label=gtk_label_new(z);
     gtk_label_set_justify(GTK_LABEL(label),GTK_JUSTIFY_LEFT);
