@@ -1,4 +1,4 @@
-/* $Id: main.cc,v 1.11 2002/02/13 03:53:56 bergo Exp $ */
+/* $Id: main.cc,v 1.12 2002/04/23 07:55:09 bergo Exp $ */
 
 /*
 
@@ -37,9 +37,12 @@
 
 static void dump_tables(int signum) {
   // dump all internal tables to stderr when SIGUSR1 is received
+  /*
   global.dumpGames();
   global.dumpBoards();
   global.dumpPanes();
+  */
+  global.report();
 }
 
 int main(int argc,char **argv) {
@@ -47,9 +50,6 @@ int main(int argc,char **argv) {
   int i;
   MainWindow *z;
 
-  if (!getenv("HOME"))
-    cerr << "** eboard ** warning: HOME environment variable not set\n";
-
   if (strstr(argv[0],"beeper")) {
     SoundSlave slave;
     slave.waitForEvents();
