@@ -1,4 +1,4 @@
-/* $Id: mainwindow.cc,v 1.91 2002/04/24 05:25:09 bergo Exp $ */
+/* $Id: mainwindow.cc,v 1.96 2002/06/28 01:33:05 bergo Exp $ */
 
 /*
 
@@ -37,7 +37,9 @@
 #include "bugpane.h"
 #include "global.h"
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #include "icon-eboard.xpm"
 
@@ -146,6 +148,9 @@ MainWindow::MainWindow() {
   adlist=0;
   consolecopy=0;
   scriptlist=0;
+  ims=0;
+
+  asetprefix.set("%%prefix *");
 
   for(i=0;i<8;i++)
     nav_enable[i]=true;
@@ -279,9 +284,14 @@ MainWindow::MainWindow() {
   gtk_signal_connect(GTK_OBJECT(inputbox),"focus_out_event",
 		     GTK_SIGNAL_FUNC(mainwindow_gcp_about_to_get_angry),0);
 
-  gtk_box_pack_start(GTK_BOX(tw[1]),inputbox,TRUE,TRUE,0);
+  ims=new InputModeSelector();
+
+  gtk_box_pack_start(GTK_BOX(tw[1]),ims->widget,FALSE,TRUE,0);
+  gtk_box_pack_start(GTK_BOX(tw[1]),inputbox,TRUE,TRUE,4);
+  
 
   gtk_widget_show(inputbox);
+  ims->show();
   gtk_widget_show(tw[0]);
   gtk_widget_show(tw[1]);
   gtk_box_pack_start(GTK_BOX(v),tw[0],FALSE,FALSE,0);
@@ -372,6 +382,7 @@ MainWindow::MainWindow() {
   global.qbcontainer=(UpdateInterface *)this;
   global.quickbar=quickbar;
   global.killbox=tw[1];
+  global.toplevelwidget=widget;
   mainw=this;
 
   notebook->setListener(this);
@@ -647,6 +658,20 @@ void MainWindow::updateBookmarks() {
     gtk_container_remove( GTK_CONTAINER(pmenu), GTK_WIDGET(s->data) );
   g_list_free(r);
 
+  if (!global.EnginePresets.empty()) {
+    item=gtk_menu_item_new_with_label("Edit Bookmarks...");
+    gtk_menu_append(GTK_MENU(pmenu),item);
+    gtk_widget_show(item);
+
+    gtk_signal_connect(GTK_OBJECT(item),"activate",
+		       GtkSignalFunc(mainwindow_edit_engbm),
+		       (gpointer) this);
+
+    item=gtk_menu_item_new();
+    gtk_menu_append(GTK_MENU(pmenu),item);
+    gtk_widget_show(item);
+  }
+
   i=1;
   for(ei=global.EnginePresets.begin();ei!=global.EnginePresets.end();ei++, i++) {
     sprintf(z,"%d. ",i);
@@ -668,7 +693,6 @@ void MainWindow::updateBookmarks() {
     gtk_menu_append(GTK_MENU(pmenu),item);
     gtk_widget_show(item);    
   }
-
 }
 
 void MainWindow::searchThemes() {
@@ -761,6 +785,15 @@ void MainWindow::greet() {
 
 }
 
+void MainWindow::updatePrefix() {
+  int id;
+  id=notebook->getCurrentPageId();
+  if (id==-2) {
+    imscache.set(-2,global.ConsoleReply);
+    ims->setPrefix( * (imscache.get(-2)) );
+  }
+}
+
 void MainWindow::setPasswordMode(int pm) {
   HideMode=pm;
   global.PauseLog=pm;
@@ -856,7 +889,29 @@ void MainWindow::userInput(char *text) {
 }
 
 void MainWindow::injectInput() {
-  userInput(gtk_entry_get_text(GTK_ENTRY(inputbox)));
+  char z[4096];
+  int id;
+
+  strcpy(z,gtk_entry_get_text(GTK_ENTRY(inputbox)));
+
+  if (asetprefix.match(z)) {
+    id=notebook->getCurrentPageId();
+    imscache.set(id, asetprefix.getStarToken(0) );
+    ims->setPrefix( * (imscache.get(id)) );
+    goto ii_nothing_else;
+  }
+
+  if (ims->getChatMode()) {
+    strcpy(z,ims->getPrefix().c_str());
+    if (z[0])
+      strcat(z," ");
+  } else
+    z[0]=0;
+  strcat(z,gtk_entry_get_text(GTK_ENTRY(inputbox)));
+
+  userInput(z);
+
+ ii_nothing_else:
   gtk_entry_set_text(GTK_ENTRY(inputbox),"\0");
 }
 
@@ -885,7 +940,7 @@ void MainWindow::saveDesk() {
   global.writeRC();  
 }
 
-void MainWindow::openServer(char *host,int port,ProtocolParser *protocol,
+void MainWindow::openServer(char *host,int port,Protocol *protocol,
 			    char *helper) {
   global.debug("MainWindow","openServer",host);
   inconsole->pop();
@@ -894,41 +949,41 @@ void MainWindow::openServer(char *host,int port,ProtocolParser *protocol,
 }
 
 void MainWindow::openXBoardEngine() {
-  XBoardProtocolParser *xpp;
+  XBoardProtocol *xpp;
   global.debug("MainWindow","openXBoardEngine");
-  xpp=new XBoardProtocolParser();
+  xpp=new XBoardProtocol();
   openEngine(xpp);
 }
 
 void MainWindow::openGnuChess4() {
-  GnuChess4ProtocolParser *gpp;
+  GnuChess4Protocol *gpp;
   global.debug("MainWindow","openGnuChess4");
-  gpp=new GnuChess4ProtocolParser();
+  gpp=new GnuChess4Protocol();
   openEngine(gpp);
 }
 
 void MainWindow::openCrafty() {
-  CraftyProtocolParser *cpp;
+  CraftyProtocol *cpp;
   global.debug("MainWindow","openCrafty");
-  cpp=new CraftyProtocolParser();
+  cpp=new CraftyProtocol();
   openEngine(cpp);
 }
 
 void MainWindow::openSjeng() {
-  SjengProtocolParser *spp;
+  SjengProtocol *spp;
   global.debug("MainWindow","openSjeng");
-  spp=new SjengProtocolParser();
+  spp=new SjengProtocol();
   openEngine(spp);  
 }
 
 void MainWindow::openEngineBookmark(EngineBookmark *bm) {
-  XBoardProtocolParser *xpp;
+  EngineProtocol *xpp;
 
   switch(bm->proto) {
-  case 0: xpp=new XBoardProtocolParser(); break;
-  case 1: xpp=new CraftyProtocolParser(); break;
-  case 2: xpp=new SjengProtocolParser(); break;
-  case 3: xpp=new GnuChess4ProtocolParser(); break;
+  case 0: xpp=new XBoardProtocol(); break;
+  case 1: xpp=new CraftyProtocol(); break;
+  case 2: xpp=new SjengProtocol(); break;
+  case 3: xpp=new GnuChess4Protocol(); break;
   default: 
     cerr << "** [eboard] bad engine protocol # in bookmark: " << bm->proto << endl;
     return;
@@ -942,17 +997,24 @@ void MainWindow::openEngineBookmark(EngineBookmark *bm) {
   }
 }
 
-void MainWindow::openEngine(XBoardProtocolParser *xpp) {
+void MainWindow::openEngine(EngineProtocol *xpp, EngineBookmark *ebm=0) {
+  int i;
   global.debug("MainWindow","openEngine");
   disconnect();
-  if (xpp->run()) {
+  
+  if (ebm)
+    i=xpp->run(ebm);
+  else
+    i=xpp->run();
+
+  if (i) {
     // success
     global.protocol=xpp;
     incrementHook();
   }
 }
 
-void MainWindow::tryConnect(char *host,int port,ProtocolParser *protocol,
+void MainWindow::tryConnect(char *host,int port,Protocol *protocol,
 			    char *helper)
 {
   DirectConnection *bytcp;
@@ -1185,6 +1247,28 @@ void MainWindow::paneChanged(int pgseq,int pgid) {
   // other boards: >= 0
   // most probably computer thinking pane will be -4
 
+  // change talking prefix as appropriate:
+
+  if ( (pgid != -2) && imscache.get(pgid)) {
+    ims->setPrefix(* (imscache.get(pgid)));
+  } else if (pgid <= -200) {
+    int ch;
+    char z[32];
+    ch=-200-pgid;
+    sprintf(z,"t %d",ch);
+    ims->setPrefix(z);
+    imscache.set(pgid, ims->getPrefix());
+  } else if (pgid >= 0) {
+    ims->setPrefix(",");
+    imscache.set(pgid, ims->getPrefix());
+  } else if (pgid == -2) {    
+    ims->setPrefix(global.ConsoleReply);
+    imscache.set(pgid, ims->getPrefix());   
+  } else if (pgid == -1) {
+    ims->setPrefix("say");
+    imscache.set(pgid, ims->getPrefix());
+  }
+
   for(i=0;i<8;i++) nv[i]=true;
 
   // 0..4
@@ -1269,6 +1353,10 @@ main_key_press (GtkWidget * wid, GdkEventKey * evt, gpointer data) {
   }
 
   switch(evt->keyval) {
+  case GDK_Escape:
+    if (ptr->ims)
+      ptr->ims->flip();
+    break;
   case GDK_F3: // previous pane
     global.ebook->goToPrevious();
     gtk_widget_grab_focus(ptr->inputbox);
@@ -1378,7 +1466,7 @@ peer_scratch_initial(gpointer data) {
 
 void
 peer_connect_fics(gpointer data) {
-  mainw->openServer("freechess.org",5000,new FicsProtocolParser(),"timeseal");
+  mainw->openServer("freechess.org",5000,new FicsProtocol(),"timeseal");
   if (global.FicsAutoLogin)
     if (global.network)
       if (global.network->isConnected())
@@ -1711,7 +1799,7 @@ mainwindow_connect_bookmark(GtkWidget *w, gpointer data) {
 
   // gcc 2.95 and up are just annoying with this const / non-const thing
   strcpy(p,"timeseal");
-  global.chandler->openServer(bm->host,bm->port,new FicsProtocolParser(),
+  global.chandler->openServer(bm->host,bm->port,new FicsProtocol(),
 			      strcmp(bm->protocol,"FICS") ? 0 : p);
 }
 
@@ -1723,6 +1811,13 @@ mainwindow_connect_bookmark2(GtkWidget *w, gpointer data) {
   mainw->openEngineBookmark(bm);
 }
 
+void
+mainwindow_edit_engbm(GtkWidget *w, gpointer data) {
+  MainWindow *me;
+  me=(MainWindow *) data;
+  (new EditEngineBookmarksDialog( (BookmarkListener *) me ))->show();
+}
+
 // ThemeEntry
 
 bool ThemeEntry::isDupe(ThemeEntry *te) {
@@ -1736,3 +1831,131 @@ bool ThemeEntry::isNameDupe(ThemeEntry *te) {
   if (!Text.compare(te->Text)) return true;
   return false;
 }
+
+// --- input mode selector
+
+InputModeSelector::InputModeSelector() {
+  GtkWidget *h;
+  int i;
+
+  ChatMode=false;
+  prefix.erase();
+
+  widget = gtk_button_new();
+
+  h=gtk_hbox_new(FALSE,0);
+  l[0]=gtk_label_new("[cmd]");
+  l[1]=gtk_label_new("");
+
+  gtk_container_add(GTK_CONTAINER(widget),h);
+  for(i=0;i<2;i++) {
+    gtk_box_pack_start(GTK_BOX(h),l[i],FALSE,TRUE,3*i);
+    gtk_widget_show(l[i]);
+  }
+
+  gtk_widget_show(h);
+
+  gtk_signal_connect(GTK_OBJECT(widget), "clicked",
+		     GTK_SIGNAL_FUNC(ims_switch), (gpointer) this);
+}
+
+bool InputModeSelector::getChatMode() {
+  return ChatMode;
+}
+
+void InputModeSelector::setChatMode(bool m) {
+  string x;
+  ChatMode=m;
+
+  x=m?"[chat]":"[cmd]";
+  gtk_label_set_text(GTK_LABEL(l[0]), x.c_str());
+
+  if (m) {
+    x=prefix;
+    setColor(l[0],0xff,0,0);
+    setColor(l[1],0,0,0xff);
+  } else {
+    x.erase();
+    setColor(l[0],0,0,0);
+  }
+
+
+
+  gtk_label_set_text(GTK_LABEL(l[1]), x.c_str());
+  gtk_widget_queue_resize(widget);
+}
+
+string & InputModeSelector::getPrefix() {
+  return prefix;
+}
+
+void InputModeSelector::setPrefix(char *s) {
+  prefix=s;
+  if (ChatMode) setChatMode(true);
+}
+
+void InputModeSelector::setPrefix(string &s) {
+  prefix=s;
+  if (ChatMode) setChatMode(true);
+}
+
+void InputModeSelector::setColor(GtkWidget *w, int R,int G, int B) {
+  GdkColor nc;
+  GtkStyle *style;
+  int i;
+
+  nc.red   = R << 8;
+  nc.green = G << 8;
+  nc.blue  = B << 8;
+  style=gtk_style_new();
+
+  for(i=0;i<5;i++)
+    style->fg[i]=nc;
+
+  gtk_widget_set_style( w, style );
+  gtk_widget_queue_draw( w );  
+}
+
+void InputModeSelector::flip() {
+  setChatMode(!ChatMode);
+}
+
+void ims_switch(GtkWidget *w, gpointer data) {
+  InputModeSelector *me;
+  me=(InputModeSelector *) data;
+  me->flip();
+}
+
+string * PrefixCache::get(int id) {
+  int i;
+  // search back to front, since we are likely to get the latest
+  // to be added
+  for(i=ids.size()-1;i>=0;i--) 
+    if (ids[i] == id) return( text[i] );
+  return 0;
+}
+
+void PrefixCache::set(int id,string &val) {
+  int i;
+  for(i=ids.size()-1;i>=0;i--) 
+    if (ids[i] == id) {
+      if (text[i]) delete(text[i]);
+      text[i]=new string(val);
+      return;
+    }
+  ids.push_back(id);
+  text.push_back( new string(val) );
+}
+
+void PrefixCache::set(int id,char *val) {
+  set( id, * (new string(val)) );
+}
+
+void PrefixCache::setIfNotSet(int id, string &val) {
+  int i;
+  for(i=ids.size()-1;i>=0;i--) 
+    if (ids[i] == id)
+      return;
+  ids.push_back(id);
+  text.push_back( new string(val) );
+}
