@@ -1,4 +1,4 @@
-/* $Id: mainwindow.h,v 1.53 2002/04/24 05:25:09 bergo Exp $ */
+/* $Id: mainwindow.h,v 1.57 2002/06/28 01:33:05 bergo Exp $ */
 
 /*
 
@@ -56,6 +56,44 @@ class ThemeEntry {
   string Text;
 };
 
+// implements the chat / command mode switch
+
+class InputModeSelector : public WidgetProxy {
+ public:
+  InputModeSelector();
+
+  bool getChatMode();
+  void setChatMode(bool m);
+  string &getPrefix();
+  void   setPrefix(char *s);
+  void   setPrefix(string &s);
+
+  void flip();
+
+ private:
+  bool   ChatMode;
+  string prefix;
+  GtkWidget *l[2];
+
+  // R,G,B in [0,255]
+  void setColor(GtkWidget *w, int R,int G, int B);
+
+  friend void ims_switch(GtkWidget *w, gpointer data);
+};
+
+class PrefixCache {
+ public:
+
+  string * get(int id);
+  void     set(int id,string &val);
+  void     set(int id,char *val);
+  void     setIfNotSet(int id, string &val);
+
+ private:
+  vector<int>      ids;
+  vector<string *> text;
+};
+
 // ask about our multiple inheritance contest
 
 class MainWindow : public WidgetProxy, 
@@ -72,7 +110,8 @@ class MainWindow : public WidgetProxy,
  public:
   MainWindow();
   void setPasswordMode(int pm);
-  void openServer(char *host,int port,ProtocolParser *protocol,char *helper);
+  void openServer(char *host,int port,Protocol *protocol,char *helper);
+  void openEngine(EngineProtocol *xpp, EngineBookmark *ebm=0);
   void openGameList();
   void openAdList();
   void openStockList();
@@ -86,6 +125,7 @@ class MainWindow : public WidgetProxy,
   void paneChanged(int pgseq,int pgid);
   void userInput(char *text);
   void peekKeys(GtkWidget *who);
+  void updatePrefix();
 
   void restoreDesk();
 
@@ -112,6 +152,10 @@ class MainWindow : public WidgetProxy,
   ScriptList      *scriptlist;
   DetachedConsole *consolecopy;
 
+  InputModeSelector *ims;
+  ExtPatternMatcher asetprefix;
+  PrefixCache       imscache;
+
   bool QuickbarVisible;
 
   GtkTooltips *tooltips;
@@ -139,11 +183,10 @@ class MainWindow : public WidgetProxy,
   void injectInput();
   void historyUp();
   void historyDown();
-  void tryConnect(char *host,int port,ProtocolParser *protocol,char *helper);
+  void tryConnect(char *host,int port,Protocol *protocol,char *helper);
   void cleanUpConnection();
   void disconnect();
 
-  void openEngine(XBoardProtocolParser *xpp);
   void openEngineBookmark(EngineBookmark *bm);
   void openGnuChess4();
   void openCrafty();
@@ -197,6 +240,7 @@ class MainWindow : public WidgetProxy,
 
   friend void     mainwindow_connect_bookmark(GtkWidget *w, gpointer data);
   friend void     mainwindow_connect_bookmark2(GtkWidget *w, gpointer data);
+  friend void     mainwindow_edit_engbm(GtkWidget *w, gpointer data);
 
   friend void     navbar_back_all(GtkWidget *w,gpointer data);
   friend void     navbar_back_1(GtkWidget *w,gpointer data);
