@@ -1,4 +1,4 @@
-/* $Id: network.h,v 1.13 2002/02/10 13:43:40 bergo Exp $ */
+/* $Id: network.h,v 1.15 2002/07/06 21:05:11 bergo Exp $ */
 
 /*
 
@@ -40,6 +40,7 @@
 #include <sys/socket.h>
 #include <netdb.h>
 #include <netinet/in.h>
+#include <netinet/tcp.h>
 
 #include <gtk/gtk.h>
 
@@ -158,6 +159,8 @@ class PipeConnection : public BufferedConnection,
   int  open();
 
   void close();
+
+  void setHandshake(char *s);
   
   // -1 if nothing available, 0 when line ok
   int readLine(char *tbuffer,int limit);
@@ -180,6 +183,7 @@ class PipeConnection : public BufferedConnection,
   vector<char *> args;
   int  pid;
   int  toid; // timeout
+  string handshake;
 };
 
 class FallBackConnection : public NetConnection {
