@@ -1,4 +1,4 @@
-/* $Id: notebook.cc,v 1.18 2002/04/07 17:24:23 bergo Exp $ */
+/* $Id: notebook.cc,v 1.19 2002/06/26 01:06:59 bergo Exp $ */
 
 /*
 
@@ -237,11 +237,10 @@ void Notebook::setTabColor(int page_num,int color,int poly,
   nc.green=(color&0x00ff00);
   nc.blue=((color&0xff)<<8);
   style=gtk_style_new();
-  style->fg[0]=nc;
-  style->fg[1]=nc;
-  style->fg[2]=nc;
-  style->fg[3]=nc;
-  style->fg[4]=nc;
+
+  for(i=0;i<5;i++)
+    style->fg[i]=nc;
+
   gtk_widget_set_style( (*pi)->label, style );
   gtk_widget_queue_draw( (*pi)->label );
 }
