@@ -1,4 +1,4 @@
-/* $Id: pieces.cc,v 1.23 2002/04/27 14:43:44 bergo Exp $ */
+/* $Id: pieces.cc,v 1.29 2002/06/17 03:24:31 bergo Exp $ */
 
 /*
 
@@ -28,8 +28,14 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
+#include <unistd.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #include "pieces.h"
 #include "global.h"
+#include "tstring.h"
+
+#include "fallback.xpm"
 
 PReq::PReq(piece a, rgbptr b, int c, int d, int e, bool f) {
   p=a; dest=b; x=c; y=d; dwidth=e; onlytop=f;
@@ -66,9 +72,11 @@ PieceSet::PieceSet(string &name, string &sqname) {
 
   isrc=gdk_imlib_load_image(cpath);
   if (isrc==0) {
-    cerr << "<PieceSet::PieceSet> ** unable to load " << wherever << endl;
-    exit(2);
+    cerr << "<PieceSet::PieceSet>";
+    cerr << " ** PNG LOAD FAILED: using internal low-res pieceset. It'll look ugly.\n";
+    isrc=FallbackLoad();
   }
+
   gdk_imlib_changed_image(isrc);
 
   // use squares from other file ?
@@ -100,7 +108,6 @@ PieceSet::PieceSet(string &name, string &sqname) {
   bgcolor|=((int)(isrc->rgb_data[2]));
   alphamask=0;
   calcAlphaMask();
-  init_cheap();
 }
 
 PieceSet::PieceSet(PieceSet *src) {
@@ -118,7 +125,6 @@ PieceSet::PieceSet(PieceSet *src) {
   memcpy(alphamask,src->alphamask,side*height*14);
   bottom_only=false;
   top_only=false;
-  init_cheap();
 }
 
 PieceSet::~PieceSet() {
@@ -126,7 +132,6 @@ PieceSet::~PieceSet() {
   if (alphamask)
     g_free(alphamask);
   gdk_imlib_destroy_image(isrc);
-  del_cheap();
   clearCache();
 
   if (!myq.empty()) {
@@ -158,10 +163,13 @@ void PieceSet::loadSquares(string &path) {
   global.debug("PieceSet","loadSquares",cpath);
 
   tmp=gdk_imlib_load_image(cpath);
+
   if (tmp==0) {
-    cerr << "<PieceSet::loadSquares> ** unable to load " << path << endl;
-    return;
+    cerr << "<PieceSet::loadSquares>";
+    cerr << " ** PNG LOAD FAILED: using internal low-res pieceset. It'll look ugly.\n";
+    tmp=FallbackLoad();
   }
+
   gdk_imlib_changed_image(tmp);
 
   w1=isrc->rgb_width;
@@ -684,6 +692,189 @@ void PieceSet::calcAlphaMask() {
   }
 }
 
+void PieceSet::drawPiece(piece p, GdkPixmap *dest, GdkGC *gc,
+			 int x, int y) {
+  int sx=0,sy=0,S;
+  int i,j,k,ai;
+
+  unsigned char buf[3072]; // enough for 1024 x N pieces
+  int bufcount;
+  rgbptr r,q,a;
+
+  p&=CUTFLAGS;
+  if (p==EMPTY) return;
+
+  if (p&BLACK) sy=height;
+  sx=side*((p&PIECE_MASK)-1);
+
+  if (extruded)
+    y-=(height-side);
+
+  S=0;
+  r=buf;
+
+  a=alphamask+7*side*sy+sx;
+  ai=6*side;
+
+  for(i=0;i<height;i++) {
+
+    r=isrc->rgb_data+3*(7*side*(i+sy)+sx); // piece
+    S=0;
+
+    for(j=0;j<side;j++) {
+      // 2 states: 0: skipping transparent
+      //           1: accumulating pixel strip
+
+      switch(S) {
+      case 0:
+	if (*a) break;
+
+	S=1;
+
+	k=j;
+	q=buf;
+	*(q++) = *(r);
+	*(q++) = *(r+1);
+	*(q++) = *(r+2);
+	bufcount=1;
+
+	break;
+      case 1:
+	if (*a) {
+	  S=0;
+	  // commit pixel strip
+	  gdk_draw_rgb_image(dest,gc,x+k,y+i,bufcount,1,
+			     GDK_RGB_DITHER_NORMAL,
+			     buf,
+			     side * 3);
+	} else {
+
+	  *(q++) = *(r);
+	  *(q++) = *(r+1);
+	  *(q++) = *(r+2);
+	  ++bufcount;
+
+	}
+
+	break;
+      }
+
+      r+=3;
+      ++a;
+
+    } // for j
+
+    if (S==1)
+      gdk_draw_rgb_image(dest,gc,x+k,y+i,bufcount,1,
+			 GDK_RGB_DITHER_NORMAL,
+			 buf,
+			 side * 3);
+
+    a+=ai;
+    
+  } // for i
+
+}
+
+void PieceSet::drawOutlinedPiece(piece p,GdkPixmap *dest,GdkGC *gc,
+				 int x,int y,bool white)
+{
+  int sx=0,sy=0,S;
+  int i,j,k,ai;
+
+  unsigned char buf[3072]; // enough for 1024 x N pieces
+  unsigned char outline[3072];
+  int bufcount;
+  rgbptr r,q,a;
+
+  p&=CUTFLAGS;
+  if (p==EMPTY) return;
+
+  if (p&BLACK) sy=height;
+  sx=side*((p&PIECE_MASK)-1);
+
+  memset(outline,white?0xff:0,3072);
+
+  if (extruded)
+    y-=(height-side);
+
+  S=0;
+  r=buf;
+
+  a=alphamask+7*side*sy+sx;
+  ai=6*side;
+
+  for(i=0;i<height;i++) {
+
+    r=isrc->rgb_data+3*(7*side*(i+sy)+sx); // piece
+    S=0;
+
+    for(j=0;j<side;j++) {
+      // 2 states: 0: skipping transparent
+      //           1: accumulating pixel strip
+
+      switch(S) {
+      case 0:
+	if (*a) break;
+
+	S=1;
+
+	k=j;
+	q=buf;
+	*(q++) = *(r);
+	*(q++) = *(r+1);
+	*(q++) = *(r+2);
+	bufcount=1;
+
+	break;
+      case 1:
+	if (*a) {
+	  S=0;
+	  // border
+	  gdk_draw_rgb_image(dest,gc,x+k-1,y+i,bufcount+2,1,
+			     GDK_RGB_DITHER_NORMAL,
+			     outline,
+			     side * 3);	  
+
+	  // commit pixel strip
+	  gdk_draw_rgb_image(dest,gc,x+k,y+i,bufcount,1,
+			     GDK_RGB_DITHER_NORMAL,
+			     buf,
+			     side * 3);
+	} else {
+
+	  *(q++) = *(r);
+	  *(q++) = *(r+1);
+	  *(q++) = *(r+2);
+	  ++bufcount;
+
+	}
+
+	break;
+      }
+
+      r+=3;
+      ++a;
+
+    } // for j
+
+    if (S==1) {
+      gdk_draw_rgb_image(dest,gc,x+k-1,y+i,bufcount+2,1,
+			 GDK_RGB_DITHER_NORMAL,
+			 outline,
+			 side * 3);	  
+      gdk_draw_rgb_image(dest,gc,x+k,y+i,bufcount,1,
+			 GDK_RGB_DITHER_NORMAL,
+			 buf,
+			 side * 3);
+    }
+
+    a+=ai;
+    
+  } // for i
+
+}
+
 void PieceSet::drawPiece(piece p,rgbptr dest,int x,int y,int dwidth) {
   int sx=0, sy=0;
   int top, sc=0, hlt=0;
@@ -829,75 +1020,6 @@ void PieceSet::rect_outline(rgbptr img,int x,int y,int w,int h,int iw) {
   }  
 }
 
-int PieceSet::cheap_sqrt(int x) {
-  return (cheap_table[x]);
-}
-
-void PieceSet::init_cheap() {
-  int i;
-  cheap_table=(int *)g_malloc(300*sizeof(int));
-  
-  for(i=0;i<300;i++)
-    cheap_table[i]=(int)(sqrt((double)i));
-}
-
-void PieceSet::del_cheap() {
-  g_free(cheap_table);
-}
-
-int PieceSet::pixr(int triplet) {
-  return((triplet>>16)&0xff);
-}
-
-int PieceSet::pixg(int triplet) {
-  return((triplet>>8)&0xff);
-}
-
-int PieceSet::pixb(int triplet) {
-  return(triplet&0xff);
-}
-
-int PieceSet::pixel(int r,int g,int b) {
-  int v=((r&0xff)<<16)|((g&0xff)<<8)|(b&0xff);
-  return(v);
-}
-
-int PieceSet::RGB2YCbCr(int triplet) {
-  float y,cb,cr;
-
-  y=0.257*(float)pixr(triplet)+
-    0.504*(float)pixg(triplet)+
-    0.098*(float)pixb(triplet)+16.0;
-
-  cb=-0.148*(float)pixr(triplet)-
-      0.291*(float)pixg(triplet)+
-      0.439*(float)pixb(triplet)+
-      128.0;
-
-  cr=0.439*(float)pixr(triplet)-
-     0.368*(float)pixg(triplet)-
-     0.071*(float)pixb(triplet)+
-     128.0;
-
-  return(pixel((int)y,(int)cb,(int)cr));
-}
-
-int PieceSet::YCbCr2RGB(int triplet) {
-  float r,g,b;
-
-  r=1.164*((float)pixr(triplet)-16.0)+
-    1.596*((float)pixb(triplet)-128.0);
-
-  g=1.164*((float)pixr(triplet)-16.0)-
-    0.813*((float)pixb(triplet)-128.0)-
-    0.392*((float)pixg(triplet)-128.0);
-  
-  b=1.164*((float)pixr(triplet)-16.0)+
-    2.017*((float)pixg(triplet)-128.0);
-
-  return(pixel((int)r,(int)g,(int)b));
-}
-
 int  PieceSet::getPixel(int x,int y) {
   rgbptr src;
   src=isrc->rgb_data+(y*(isrc->rgb_width)+x)*3;
@@ -956,6 +1078,50 @@ void PieceSet::endQueueing() {
   top_only=false;
 }
 
+
+GdkImlibImage * PieceSet::FallbackLoad() {
+  tstring t;
+  int w,h,c,i,j;
+  rgbptr data, p;
+  GdkImlibImage *img;
+  string *s;
+
+  int cmap[256];
+  int rgb;
+  
+  t.set(fallback_xpm[0]);
+
+  w=t.tokenvalue(" \t");
+  h=t.tokenvalue(" \t");
+  c=t.tokenvalue(" \t");
+
+  for(i=0;i<c;i++) {
+    t.set(fallback_xpm[i+1]);
+    t.token("\t");
+    t.token(" ");
+    s=t.token("# \t");
+    if (s)
+      rgb=strtol(s->c_str(),0,16);
+    else
+      rgb=0;
+    cmap[fallback_xpm[i+1][0]]=rgb;
+  }
+
+  data=(rgbptr)g_malloc(w*h*3);
+
+  p=data;
+  for(j=0;j<h;j++)
+    for(i=0;i<w;i++) {
+      *(p++)=pixr(cmap[fallback_xpm[1+c+j][i]]);
+      *(p++)=pixg(cmap[fallback_xpm[1+c+j][i]]);
+      *(p++)=pixb(cmap[fallback_xpm[1+c+j][i]]);
+    }
+
+  img=gdk_imlib_create_image_from_data(data,0,w,h);
+  g_free(data);
+  return img;
+}
+
 // ------------------------------ vectorized pieces
 
 // offsets 32 67 90 76 65 46 
@@ -1128,3 +1294,76 @@ void VecDetail::draw(GdkPixmap *dest, GdkGC *gc,int x,int y,int sz1,int sz2) {
 		  x+(x1[i]*sz1)/sz2,y+(y1[i]*sz1)/sz2,
 		  x+(x2[i]*sz1)/sz2,y+(y2[i]*sz1)/sz2);
 }
+
+// ----------- RGB <-> YCbCr transform
+
+int ColorSpace::lighter(int triplet) {
+  int a,b;
+  a=RGB2YCbCr(triplet);
+  b=pixr(a);
+  b=(256+b)/2;
+  b=YCbCr2RGB(pixel(b,pixg(a),pixb(a)));
+  return b;
+}
+
+int ColorSpace::darker(int triplet) {
+  int a,b;
+  a=RGB2YCbCr(triplet);
+  b=pixr(a);
+  b/=2;
+  b=YCbCr2RGB(pixel(b,pixg(a),pixb(a)));
+  return b;
+}
+
+int ColorSpace::RGB2YCbCr(int triplet) {
+  float y,cb,cr;
+
+  y=0.257*(float)pixr(triplet)+
+    0.504*(float)pixg(triplet)+
+    0.098*(float)pixb(triplet)+16.0;
+
+  cb=-0.148*(float)pixr(triplet)-
+      0.291*(float)pixg(triplet)+
+      0.439*(float)pixb(triplet)+
+      128.0;
+
+  cr=0.439*(float)pixr(triplet)-
+     0.368*(float)pixg(triplet)-
+     0.071*(float)pixb(triplet)+
+     128.0;
+
+  return(pixel((int)y,(int)cb,(int)cr));
+}
+
+int ColorSpace::YCbCr2RGB(int triplet) {
+  float r,g,b;
+
+  r=1.164*((float)pixr(triplet)-16.0)+
+    1.596*((float)pixb(triplet)-128.0);
+
+  g=1.164*((float)pixr(triplet)-16.0)-
+    0.813*((float)pixb(triplet)-128.0)-
+    0.392*((float)pixg(triplet)-128.0);
+  
+  b=1.164*((float)pixr(triplet)-16.0)+
+    2.017*((float)pixg(triplet)-128.0);
+
+  return(pixel((int)r,(int)g,(int)b));
+}
+
+int ColorSpace::pixr(int triplet) {
+  return((triplet>>16)&0xff);
+}
+
+int ColorSpace::pixg(int triplet) {
+  return((triplet>>8)&0xff);
+}
+
+int ColorSpace::pixb(int triplet) {
+  return(triplet&0xff);
+}
+
+int ColorSpace::pixel(int r,int g,int b) {
+  int v=((r&0xff)<<16)|((g&0xff)<<8)|(b&0xff);
+  return(v);
+}
