@@ -1,4 +1,4 @@
-/* $Id: pieces.h,v 1.17 2002/04/27 14:43:44 bergo Exp $ */
+/* $Id: pieces.h,v 1.22 2002/06/17 03:24:31 bergo Exp $ */
 
 /*
 
@@ -47,7 +47,22 @@ class PReq {
   bool   onlytop;
 };
 
-class PieceSet {
+class ColorSpace {
+ public:
+
+  int RGB2YCbCr(int triplet);
+  int YCbCr2RGB(int triplet);
+  int lighter(int triplet);
+  int darker(int triplet);
+
+  int pixr(int triplet);
+  int pixg(int triplet);
+  int pixb(int triplet);
+  int pixel(int r,int g,int b);
+
+};
+
+class PieceSet : public ColorSpace {
  public:
   PieceSet(string &name, string &sqname); 
   PieceSet(PieceSet *src);
@@ -70,6 +85,12 @@ class PieceSet {
   void drawPieceAndSquare(piece p,rgbptr dest,int x,int y,int dwidth,
 			  int sqdark);
 
+  // fast fully masked draw
+  void drawPiece(piece p,GdkPixmap *dest,GdkGC *gc,int x,int y);
+  
+  void drawOutlinedPiece(piece p,GdkPixmap *dest,GdkGC *gc,
+			 int x,int y,bool white);
+
   // for extruded sets
   void beginQueueing();
   void endQueueing();
@@ -93,6 +114,8 @@ class PieceSet {
   void buildCache();
   void clearCache();
 
+  GdkImlibImage *FallbackLoad();
+
   void loadSquares(string &path);
 
   void calcAlphaMask();
@@ -106,22 +129,8 @@ class PieceSet {
   void xSetPixel(rgbptr img,int x,int y,int v);
   int  alpha(int x,int y);
 
-  int pixr(int triplet);
-  int pixg(int triplet);
-  int pixb(int triplet);
-  int pixel(int r,int g,int b);
-  int RGB2YCbCr(int triplet);
-  int YCbCr2RGB(int triplet);
-
   void queueRequest(PReq *r);
 
-  int * cheap_table;
-
-  void init_cheap();
-  void del_cheap();
-
-  int cheap_sqrt(int x);
-
   GdkImlibImage * gimp_scale_preserve(GdkImlibImage *src,
 				      rgbptr alpha,
 				      int nw,int nh);
