@@ -1,4 +1,4 @@
-/* $Id: position.cc,v 1.39 2002/05/02 00:59:23 bergo Exp $ */
+/* $Id: position.cc,v 1.40 2002/06/14 17:32:57 bergo Exp $ */
 
 /*
 
@@ -160,6 +160,7 @@ void Position::setStartPos() {
     maycastle[i]=true;
   ep[0]=-1;
   clearStock();
+  sidehint=true;
 }
 
 int Position::operator==(Position &p) {
@@ -201,6 +202,8 @@ Position Position::operator=(Position &p) {
     global.annotator.link(*ai);
   }
 
+  sidehint=p.sidehint;
+
   return(*this);
 }
 
@@ -259,6 +262,9 @@ void Position::moveAnyNotation(char *m,piece color,variant Vr=REGULAR) {
   piece pr;
   char xlate[12];
   global.debug("Position","moveAnyNotation",m);
+
+  if (color==WHITE) sidehint=false; else sidehint=true;
+
   if (m[1]=='@') {
     switch(m[0]) {
     case 'Q':
@@ -319,6 +325,8 @@ void Position::moveStdNotation(char *m,piece color,variant Vr=REGULAR) {
   int cy;
   global.debug("Position","moveStdNotation",m);
 
+  if (color==WHITE) sidehint=false; else sidehint=true;
+
   what=PAWN;
   from[0]=from[1]=to[0]=to[1]=-1;
   istake=0;
@@ -560,6 +568,9 @@ void Position::moveCartesian(int x1,int y1,int x2,int y2,
 {
   piece color;
 
+  color=square[x1][y1]&COLOR_MASK;
+  if (color==WHITE) sidehint=false; else sidehint=true;
+
   // castling
   if ((x1==4)&&(x2==6)&&(!(y1%7))&&(!(y2%7))&&
       ( (square[x1][y1]&PIECE_MASK)==KING) ) {
@@ -575,7 +586,6 @@ void Position::moveCartesian(int x1,int y1,int x2,int y2,
   // en passant (calling moveStdNotation for it would cause an infinite loop)
   if ( ( (square[x1][y1]&PIECE_MASK) == PAWN ) && (square[x2][y2]==EMPTY)&&
       (x1!=x2)) {
-    color=square[x1][y1]&COLOR_MASK;
     square[x2][y2+((color==WHITE)?-1:1)]=EMPTY;
     if (ep[0]>=0) ep[0]=-1;
   }
@@ -1290,7 +1300,11 @@ string & Position::getFEN() {
     if (i>0) fen+='/';
 
   }
-  fen+=" w "; // FIXME, but for start positions will do
+
+  if (sidehint)
+    fen+=" w ";
+  else
+    fen+=" b ";
   
   ec=0;
   if (maycastle[0]) { fen+="K"; ++ec; }
@@ -1319,8 +1333,12 @@ void Position::setFEN(const char *fen) {
     for(r=0;r<8;r++)
       square[c][r]=EMPTY;
 
+  p=fen;
+  while(*p==' ') ++p;
+  if (! *p) return;
+
   c=0; r=7;
-  for(p=fen;(*p)!=' ';p++) {
+  for(;(*p)!=' ';p++) {
     z=*p;
 
     if (z==0) return;
@@ -1345,10 +1363,17 @@ void Position::setFEN(const char *fen) {
     square[c++][r]=pp;
   }
 
-  if (strlen(p)<5)
+  if (strlen(p)<2)
     return;
   
-  p+=3;
+  while(*p==' ') ++p;
+  if (! *p) return;
+
+  if (toupper(*p) == 'W') sidehint=true; else sidehint=false;
+
+  while(*p==' ') ++p;
+  if (! *p) return;
+
   for(c=0;c<4;c++)
     maycastle[c]=false;
   if ((*p)!='-') {
@@ -1358,6 +1383,7 @@ void Position::setFEN(const char *fen) {
       case 'Q': maycastle[2]=true; break;
       case 'k': maycastle[1]=true; break;
       case 'q': maycastle[3]=true; break;
+      case 0: return;
       }
     }
   } else
@@ -1366,7 +1392,7 @@ void Position::setFEN(const char *fen) {
   if (strlen(p)<2)
     return;
   ++p;
-  
+
   ep[0]=-1;
   if ((*p)!='-') {
     ep[0]=(*p++)-'a';
