@@ -1,4 +1,4 @@
-/* $Id: proto_crafty.h,v 1.11 2001/04/30 07:36:01 bergo Exp $ */
+/* $Id: proto_crafty.h,v 1.12 2002/06/14 17:32:57 bergo Exp $ */
 
 /*
 
@@ -28,9 +28,9 @@
 #ifndef EBOARD_CRAFTY_H
 #define EBOARD_CRAFTY_H 1
 
-class CraftyProtocolParser : public XBoardProtocolParser {
+class CraftyProtocol : public XBoardProtocol {
  public:
-  CraftyProtocolParser();
+  CraftyProtocol();
 
   virtual void initEngine();
   virtual void readDialog();
