@@ -1,4 +1,4 @@
-/* $Id: proto_fics.cc,v 1.77 2002/04/28 21:04:34 bergo Exp $ */
+/* $Id: proto_fics.cc,v 1.80 2002/06/30 21:57:47 bergo Exp $ */
 
 /*
 
@@ -26,6 +26,11 @@
 #include <iostream.h>
 #include <stdlib.h>
 #include <string.h>
+
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+
 #include <ctype.h>
 #include "protocol.h"
 #include "global.h"
@@ -35,10 +40,10 @@
 
 // <b1> game 45 white [PPPPPNBBR] black [PPNN]
 
-FicsProtocolParser::FicsProtocolParser() {
+FicsProtocol::FicsProtocol() {
   SetPat *sp;
 
-  global.debug("FicsProtocolParser","FicsProtocolParser");
+  global.debug("FicsProtocol","FicsProtocol");
 
   GActions.push_back(new string("All Observers"));
   GActions.push_back(new string("Game Info"));
@@ -196,23 +201,23 @@ FicsProtocolParser::FicsProtocolParser() {
 	     &LectureBotXTellContinued,&GotPartnerGame,0);
 }
 
-FicsProtocolParser::~FicsProtocolParser() {
-  global.debug("FicsProtocolParser","~FicsProtocolParser");
+FicsProtocol::~FicsProtocol() {
+  global.debug("FicsProtocol","~FicsProtocol");
   clearMoveBuffer();
 }
 
-void FicsProtocolParser::finalize() {
-  global.debug("FicsProtocolParser","finalize");
+void FicsProtocol::finalize() {
+  global.debug("FicsProtocol","finalize");
   if (global.network)
     global.network->writeLine("quit");  
 }
 
-void FicsProtocolParser::receiveString(char *netstring) {
+void FicsProtocol::receiveString(char *netstring) {
     int msgcolor=global.Colors.TextBright, ispersonal=0;
     char z[128];
     char pstring[1024];
 
-    global.debug("FicsProtocolParser","receiveString",netstring);
+    global.debug("FicsProtocol","receiveString",netstring);
 
     // eat timeseal annoying blank lines
     if ((netstring[0]==0)&&(LastWasStyle12)) {
@@ -263,7 +268,7 @@ void FicsProtocolParser::receiveString(char *netstring) {
 	AuthenticationGone=1;
 	strcpy(nick,WelcomeMessage.getToken(3));	
 	UserName.append(new KleeneStarPat());
-	UserName.append(new ExactStringPat(nick));
+	UserName.append(new CIExactStringPat(nick));
 	UserName.append(new KleeneStarPat());
 
 	sprintf(z,"set interface eboard %s/%s\niset startpos 1\niset defprompt 1",
@@ -460,6 +465,27 @@ void FicsProtocolParser::receiveString(char *netstring) {
       global.challenged();
     }
 
+    bool gottell;
+    ExtPatternMatcher *tm;
+
+    gottell=false;
+
+    if (PrivateTell.match()) { gottell=true; tm=&PrivateTell; }
+    if (Say.match())         { gottell=true; tm=&Say; }
+    if (Say2.match())        { gottell=true; tm=&Say2; }
+
+    if (gottell) {
+      string x;
+      char xn[64], *xp;
+      x="t ";
+      strcpy(xn,tm->getAToken(0));
+      xp=index(xn,'('); if (xp) *xp=0;
+      xp=index(xn,'['); if (xp) *xp=0;
+      x+= xn;
+      global.ConsoleReply=x;
+      global.input->updatePrefix();
+    }
+
     if (UserName.match())    {
       msgcolor=global.Colors.PrivateTell; ispersonal=1;
       if (ChannelTell.match() || LectureBotSpecial || LectureBotXTell.match() )
@@ -571,12 +597,12 @@ void FicsProtocolParser::receiveString(char *netstring) {
     LastColor=msgcolor;
 }
 
-void FicsProtocolParser::createExaminedGame(int gameid,char *whitep,char *blackp) {
+void FicsProtocol::createExaminedGame(int gameid,char *whitep,char *blackp) {
   ChessGame *cg;
   Board *b;
   char *p,z[64];
 
-  global.debug("FicsProtocolParser","createExaminedGame");
+  global.debug("FicsProtocol","createExaminedGame");
 
   cg=new ChessGame();
   cg->clock_regressive=1;
@@ -613,13 +639,13 @@ void FicsProtocolParser::createExaminedGame(int gameid,char *whitep,char *blackp
 }
 
 // creates a non-moveable, non-editable position for sposition
-void FicsProtocolParser::createScratchPos(int gameid,char *whitep,char *blackp)
+void FicsProtocol::createScratchPos(int gameid,char *whitep,char *blackp)
 {
   ChessGame *cg;
   Board *b;
   char *p,z[64];
 
-  global.debug("FicsProtocolParser","createScratchPos");
+  global.debug("FicsProtocol","createScratchPos");
 
   cg=new ChessGame();
   cg->clock_regressive=1;
@@ -646,11 +672,11 @@ void FicsProtocolParser::createScratchPos(int gameid,char *whitep,char *blackp)
   cg->acknowledgeInfo();
 }
 
-void FicsProtocolParser::createGame(ExtPatternMatcher &pm) {
+void FicsProtocol::createGame(ExtPatternMatcher &pm) {
   ChessGame *cg,*rep;
   char *p;
 
-  global.debug("FicsProtocolParser","createGame");
+  global.debug("FicsProtocol","createGame");
 
   cg=new ChessGame();
 
@@ -713,13 +739,13 @@ void FicsProtocolParser::createGame(ExtPatternMatcher &pm) {
   global.status->setText("Game started!");
 }
 
-void FicsProtocolParser::addGame(ExtPatternMatcher &pm) {
+void FicsProtocol::addGame(ExtPatternMatcher &pm) {
   ChessGame *cg;
   Board *b;
   bool is_partner_game=false;
   char tab[96];
 
-  global.debug("FicsProtocolParser","addGame");
+  global.debug("FicsProtocol","addGame");
 
   cg=new ChessGame();
 
@@ -764,12 +790,12 @@ void FicsProtocolParser::addGame(ExtPatternMatcher &pm) {
   PendingStatus++;
 }
 
-void FicsProtocolParser::discardGame(int gameid) {
+void FicsProtocol::discardGame(int gameid) {
   ChessGame *cg;
   Board *b;
   char z[64];
 
-  global.debug("FicsProtocolParser","discardGame");
+  global.debug("FicsProtocol","discardGame");
 
   cg=global.getGame(gameid);
   if (!cg)
@@ -804,15 +830,15 @@ void FicsProtocolParser::discardGame(int gameid) {
 
 }
 
-void FicsProtocolParser::removeGame(ExtPatternMatcher &pm) {
+void FicsProtocol::removeGame(ExtPatternMatcher &pm) {
   int gm;
-  global.debug("FicsProtocolParser","removeGame");
+  global.debug("FicsProtocol","removeGame");
 
   gm=atoi(pm.getNToken(0));
   innerRemoveGame(gm);
 }
 
-void FicsProtocolParser::innerRemoveGame(int gameid) {
+void FicsProtocol::innerRemoveGame(int gameid) {
   Board *b;
   ChessGame *cg;
 
@@ -833,11 +859,11 @@ void FicsProtocolParser::innerRemoveGame(int gameid) {
   cg->setBoard(0);
 }
 
-void FicsProtocolParser::sendDrop(piece p,int x,int y) {
+void FicsProtocol::sendDrop(piece p,int x,int y) {
   piece k;
   char drop[5];
 
-  global.debug("FicsProtocolParser","sendDrop");
+  global.debug("FicsProtocol","sendDrop");
 
   k=p&PIECE_MASK;
   drop[4]=0;
@@ -855,11 +881,11 @@ void FicsProtocolParser::sendDrop(piece p,int x,int y) {
   global.network->writeLine(drop);
 }
 
-void FicsProtocolParser::sendMove(int x1,int y1,int x2,int y2,int prom) {
+void FicsProtocol::sendMove(int x1,int y1,int x2,int y2,int prom) {
   char move[7];
   piece pp;
 
-  global.debug("FicsProtocolParser","sendMove");
+  global.debug("FicsProtocol","sendMove");
 
   move[4]=0;
   move[5]=0;
@@ -884,13 +910,13 @@ void FicsProtocolParser::sendMove(int x1,int y1,int x2,int y2,int prom) {
 }
 
 // attach stock info to bug/zhouse games
-void FicsProtocolParser::attachHouseStock() {
+void FicsProtocol::attachHouseStock() {
   ChessGame *cg;
   int gid;
   char *p;
   Position *pos;
 
-  global.debug("FicsProtocolParser","attachHouseStock");
+  global.debug("FicsProtocol","attachHouseStock");
 
   //  <b1> game %n white [*] black [*]*
 
@@ -920,7 +946,7 @@ void FicsProtocolParser::attachHouseStock() {
   cg->updateStock();
 }
 
-void FicsProtocolParser::parseStyle12(char *data) {
+void FicsProtocol::parseStyle12(char *data) {
   static char *sep=" \t\n";
   tstring t;
   int i,j,n;
@@ -937,7 +963,7 @@ void FicsProtocolParser::parseStyle12(char *data) {
   int ackp=0;
   bool spawnExamination;
 
-  global.debug("FicsProtocolParser","parseStyle12");
+  global.debug("FicsProtocol","parseStyle12");
 
   t.set(data);
   t.token(sep); // <12>
@@ -1113,12 +1139,12 @@ void FicsProtocolParser::parseStyle12(char *data) {
   if (rel==-1) refgame->enableMoving(false);
 }
 
-void FicsProtocolParser::updateGame(ExtPatternMatcher &pm) {
+void FicsProtocol::updateGame(ExtPatternMatcher &pm) {
   ChessGame *refgame;
   char *p,z[32];
   int gm;
 
-  global.debug("FicsProtocolParser","updateGame");
+  global.debug("FicsProtocol","updateGame");
 
   // Ns: 0=game 1=rat.white 2=rat.black
   // Ss: 0=whitep 1=blackp
@@ -1185,12 +1211,12 @@ void FicsProtocolParser::updateGame(ExtPatternMatcher &pm) {
   }
 }
 
-void FicsProtocolParser::gameOver(ExtPatternMatcher &pm, GameResult result) {
+void FicsProtocol::gameOver(ExtPatternMatcher &pm, GameResult result) {
   int game;
   char reason[64];
   ChessGame *refgame;
 
-  global.debug("FicsProtocolParser","gameOver");
+  global.debug("FicsProtocol","gameOver");
   
   game=atoi(pm.getNToken(0));
   reason[63]=0;
@@ -1231,51 +1257,51 @@ void FicsProtocolParser::gameOver(ExtPatternMatcher &pm, GameResult result) {
   }
 }
 
-void FicsProtocolParser::refreshSeeks() {
+void FicsProtocol::refreshSeeks() {
 	if ((global.IcsSeekGraph)&&(global.skgraph)) {
 		global.skgraph->clear();
 		global.network->writeLine("iset seekinfo 1");
 	}
 }
 
-int  FicsProtocolParser::hasAuthenticationPrompts() {
+int  FicsProtocol::hasAuthenticationPrompts() {
   return 1;
 }
 
-void FicsProtocolParser::resign() {
+void FicsProtocol::resign() {
   global.network->writeLine("resign");
   global.status->setText("Resigned.");
 }
 
-void FicsProtocolParser::draw() {
+void FicsProtocol::draw() {
   global.network->writeLine("draw");
   global.status->setText("<Fics Protocol> draw request sent");
 }
 
-void FicsProtocolParser::adjourn() {
+void FicsProtocol::adjourn() {
   global.network->writeLine("adjourn");
   global.status->setText("<Fics Protocol> adjourn request sent");
 }
 
-void FicsProtocolParser::abort() {
+void FicsProtocol::abort() {
   global.network->writeLine("abort");
   global.status->setText("<Fics Protocol> abort request sent");
 }
 
-void FicsProtocolParser::clearMoveBuffer() {
+void FicsProtocol::clearMoveBuffer() {
   list<Position *>::iterator li;
 
-  global.debug("FicsProtocolParser","clearMoveBuffer");
+  global.debug("FicsProtocol","clearMoveBuffer");
   MoveBuf.clear();
 }
 
-void FicsProtocolParser::retrieve1(ExtPatternMatcher &pm) {
+void FicsProtocol::retrieve1(ExtPatternMatcher &pm) {
   char z[32],mv[16];
   char tmp[2][32];
   int itmp;
   Position p,q;
 
-  global.debug("FicsProtocolParser","retrieve1");
+  global.debug("FicsProtocol","retrieve1");
 
   if (!MoveBuf.empty())
     p=MoveBuf.back();
@@ -1307,13 +1333,13 @@ void FicsProtocolParser::retrieve1(ExtPatternMatcher &pm) {
   MoveBuf.push_back(q);
 }
 
-void FicsProtocolParser::retrieve2(ExtPatternMatcher &pm) {
+void FicsProtocol::retrieve2(ExtPatternMatcher &pm) {
   Position p;
   char z[32],mv[16];
   char tmp[2][32];
   int itmp;
 
-  global.debug("FicsProtocolParser","retrieve2");
+  global.debug("FicsProtocol","retrieve2");
 
   if (!MoveBuf.empty())
     p=MoveBuf.back();
@@ -1330,12 +1356,12 @@ void FicsProtocolParser::retrieve2(ExtPatternMatcher &pm) {
   MoveBuf.push_back(p);
 }
 
-void FicsProtocolParser::retrievef() {
+void FicsProtocol::retrievef() {
   int gm;
   ChessGame *cg;
   char *p;
 
-  global.debug("FicsProtocolParser","retrievef");
+  global.debug("FicsProtocol","retrievef");
 
   if (MoveRetrievals.empty())
     return;
@@ -1360,7 +1386,7 @@ void FicsProtocolParser::retrievef() {
   clearMoveBuffer();
 }
 
-void FicsProtocolParser::queryGameList(GameListConsumer *glc) {
+void FicsProtocol::queryGameList(GameListConsumer *glc) {
   if (RetrievingGameList) {
     if (glc!=lastGLC)
       glc->endOfList();
@@ -1372,7 +1398,7 @@ void FicsProtocolParser::queryGameList(GameListConsumer *glc) {
   global.network->writeLine("games");  
 }
 
-void FicsProtocolParser::queryAdList(GameListConsumer *glc) {
+void FicsProtocol::queryAdList(GameListConsumer *glc) {
   if (RetrievingAdList) {
     if (glc!=lastALC)
       glc->endOfList();
@@ -1383,7 +1409,7 @@ void FicsProtocolParser::queryAdList(GameListConsumer *glc) {
   global.network->writeLine("sough");
 }
 
-void FicsProtocolParser::sendGameListEntry() {
+void FicsProtocol::sendGameListEntry() {
   char frank[256];
   int gn;
   //  GameListEntry.set("*%n *[*]*:*");
@@ -1401,7 +1427,7 @@ void FicsProtocolParser::sendGameListEntry() {
   lastGLC->appendGame(gn,frank);
 }
 
-void FicsProtocolParser::sendAdListEntry(char *pat) {
+void FicsProtocol::sendAdListEntry(char *pat) {
   ExtPatternMatcher rpat;
   int gn;
 
@@ -1413,19 +1439,19 @@ void FicsProtocolParser::sendAdListEntry(char *pat) {
   lastALC->appendGame(gn,rpat.getStarToken(1));
 }
 
-void FicsProtocolParser::observe(int gameid) {
+void FicsProtocol::observe(int gameid) {
   char z[64];
   sprintf(z,"observe %d",gameid);
   global.network->writeLine(z);
 }
 
-void FicsProtocolParser::answerAd(int adid) {
+void FicsProtocol::answerAd(int adid) {
   char z[64];
   sprintf(z,"play %d",adid);
   global.network->writeLine(z);
 }
 
-char * FicsProtocolParser::knowRating(char *player) {
+char * FicsProtocol::knowRating(char *player) {
   int i;
   for(i=0;i<2;i++)
     if (strlen(xxplayer[i]))
@@ -1434,28 +1460,28 @@ char * FicsProtocolParser::knowRating(char *player) {
   return 0;
 }
 
-void FicsProtocolParser::clearRatingBuffer() {
+void FicsProtocol::clearRatingBuffer() {
   xxplayer[0][0]=0;
   xxplayer[1][0]=0;
 }
 
-void FicsProtocolParser::exaForward(int n) {
+void FicsProtocol::exaForward(int n) {
   char z[64];
   sprintf(z,"forward %d",n);
   global.network->writeLine(z);
 }
 
-void FicsProtocolParser::exaBackward(int n) {
+void FicsProtocol::exaBackward(int n) {
   char z[64];
   sprintf(z,"backward %d",n);
   global.network->writeLine(z);
 }
 
-void FicsProtocolParser::retrieveMoves(ExtPatternMatcher &pm) {
+void FicsProtocol::retrieveMoves(ExtPatternMatcher &pm) {
   ChessGame *refgame;
   int gm;
   char z[64];
-  global.debug("FicsProtocolParser","retrieveMoves");
+  global.debug("FicsProtocol","retrieveMoves");
 
   gm=atoi(pm.getNToken(0));  
   refgame=global.getGame(gm);
@@ -1472,7 +1498,7 @@ void FicsProtocolParser::retrieveMoves(ExtPatternMatcher &pm) {
 
 // --- seek graph
 
-void FicsProtocolParser::ensureSeekGraph() {
+void FicsProtocol::ensureSeekGraph() {
   if (!global.skgraph) {
     global.skgraph=new SeekGraph();
     global.skgraph->show();
@@ -1481,12 +1507,12 @@ void FicsProtocolParser::ensureSeekGraph() {
   }
 }
 
-void FicsProtocolParser::seekAdd(ExtPatternMatcher &pm) {
+void FicsProtocol::seekAdd(ExtPatternMatcher &pm) {
   SeekAd *ad;
   char *p;
   int flags;
 
-  global.debug("FicsProtocolParser","seekAdd");
+  global.debug("FicsProtocol","seekAdd");
 
   // <s> 8 w=visar ti=02 rt=2194  t=4 i=0 r=r tp=suicide c=? rr=0-9999 a=t f=f
   //    n0   s0       r0    n1  *0  n2  n3  s1    r1       r2    r3      s2  s3
@@ -1540,7 +1566,7 @@ void FicsProtocolParser::seekAdd(ExtPatternMatcher &pm) {
   global.skgraph->add(ad);
 }
 
-void FicsProtocolParser::seekRemove(ExtPatternMatcher &pm) {
+void FicsProtocol::seekRemove(ExtPatternMatcher &pm) {
   tstring t;
   int i;
   t.setFail(-1);
@@ -1550,25 +1576,25 @@ void FicsProtocolParser::seekRemove(ExtPatternMatcher &pm) {
     global.skgraph->remove(i);
 }
 
-vector<string *> * FicsProtocolParser::getPlayerActions() {
+vector<string *> * FicsProtocol::getPlayerActions() {
   return(&PActions);
 }
 
-vector<string *> * FicsProtocolParser::getGameActions() {
+vector<string *> * FicsProtocol::getGameActions() {
   return(&GActions);
 }
 
-void FicsProtocolParser::callPlayerAction(char *player, string *action) {
+void FicsProtocol::callPlayerAction(char *player, string *action) {
   for(int i=0;i<PActions.size();i++)
     if ( (*PActions[i]) == (*action) ) { doPlayerAction(player, i); break; }
 }
 
-void FicsProtocolParser::callGameAction(int gameid, string *action) {
+void FicsProtocol::callGameAction(int gameid, string *action) {
   for(int i=0;i<GActions.size();i++)
     if ( (*GActions[i]) == (*action) ) { doGameAction(gameid, i); break; }
 }
 
-void FicsProtocolParser::doPlayerAction(char *player, int id) {
+void FicsProtocol::doPlayerAction(char *player, int id) {
   char z[256],w[256];
   bool ok=true;
 
@@ -1588,7 +1614,7 @@ void FicsProtocolParser::doPlayerAction(char *player, int id) {
   }
 }
 
-void FicsProtocolParser::doGameAction(int gameid, int id) {
+void FicsProtocol::doGameAction(int gameid, int id) {
   char z[256], w[256];
   bool ok=true;
 
@@ -1605,7 +1631,7 @@ void FicsProtocolParser::doGameAction(int gameid, int id) {
   }
 }
 
-void FicsProtocolParser::prepareBugPane() {
+void FicsProtocol::prepareBugPane() {
   char z[64];
   PartnerGame = atoi(GotPartnerGame.getNToken(0));
   sprintf(z,"observe %d",PartnerGame);
