@@ -1,4 +1,4 @@
-/* $Id: proto_fics.h,v 1.35 2002/04/28 21:04:34 bergo Exp $ */
+/* $Id: proto_fics.h,v 1.36 2002/06/14 17:32:57 bergo Exp $ */
 
 /*
 
@@ -31,10 +31,10 @@
 #include "stl.h"
 #include "position.h"
 
-class FicsProtocolParser : public ProtocolParser {
+class FicsProtocol : public Protocol {
  public:
-  FicsProtocolParser();
-  ~FicsProtocolParser();
+  FicsProtocol();
+  ~FicsProtocol();
 
   void receiveString(char *netstring);
   void sendMove(int x1,int y1,int x2,int y2,int prom);
