@@ -1,4 +1,4 @@
-/* $Id: proto_gnuchess4.h,v 1.2 2001/05/01 02:11:15 bergo Exp $ */
+/* $Id: proto_gnuchess4.h,v 1.3 2002/06/14 17:32:57 bergo Exp $ */
 
 /*
 
@@ -28,9 +28,9 @@
 #ifndef EBOARD_GNUCHESS4
 #define EBOARD_GNUCHESS4
 
-class GnuChess4ProtocolParser : public XBoardProtocolParser {
+class GnuChess4Protocol : public XBoardProtocol {
  public:
-  GnuChess4ProtocolParser();
+  GnuChess4Protocol();
 
   virtual void initEngine();
   virtual void readDialog();
