@@ -1,4 +1,4 @@
-/* $Id: proto_sjeng.h,v 1.1 2001/05/01 20:35:07 bergo Exp $ */
+/* $Id: proto_sjeng.h,v 1.2 2002/06/14 17:32:57 bergo Exp $ */
 
 /*
 
@@ -29,9 +29,9 @@
 #define EBOARD_SJENG
 
 
-class SjengProtocolParser : public XBoardProtocolParser {
+class SjengProtocol : public XBoardProtocol {
  public:
-  SjengProtocolParser();
+  SjengProtocol();
 
   virtual void sendDrop(piece p,int x,int y);
   virtual void initEngine();
