@@ -1,4 +1,4 @@
-/* $Id: proto_xboard.cc,v 1.22 2002/05/02 00:41:04 bergo Exp $ */
+/* $Id: proto_xboard.cc,v 1.25 2002/06/17 03:24:31 bergo Exp $ */
 
 /*
 
@@ -40,12 +40,18 @@
 #include "stl.h"
 #include "tstring.h"
 
-XBoardProtocolParser::XBoardProtocolParser() {
+XBoardProtocol::XBoardProtocol() {
   EngineWhite=0;
   MoveNumber=1;
   MoveBlack=0;
   Variant=REGULAR;
   want_path_pane=1;
+  supports_setboard=false;
+  timeout_id=-1;
+  need_handshake=true; // send xboard/protover 2
+
+  got_init_pos=false;
+
   strcpy(ComputerName,"Computer");
   strcpy(EngineCommandLine,"gnuchess");
   strcpy(EngineRunDir,"~/.eboard/eng-out");
@@ -80,8 +86,15 @@ XBoardProtocolParser::XBoardProtocolParser() {
   ebm=0;
 }
 
+void XBoardProtocol::setInitialPosition(Position *p) {
+
+  initpos=(*p);
+  got_init_pos=true;
+
+}
+
 // the "dialect" for game ends
-char * XBoardProtocolParser::xlateDialect(char *s) {
+char * XBoardProtocol::xlateDialect(char *s) {
   static char result[128];
 
   if (Dialect[0].match(s)) { strcpy(result,"0-1 { White resigns }"); return result; }
@@ -97,7 +110,7 @@ char * XBoardProtocolParser::xlateDialect(char *s) {
   return 0;
 }
 
-void XBoardProtocolParser::receiveString(char *netstring) {
+void XBoardProtocol::receiveString(char *netstring) {
   ChessGame *refgame;
   char *p;
   char a[64],b[64];
@@ -113,6 +126,16 @@ void XBoardProtocolParser::receiveString(char *netstring) {
   }
 
   if (IllegalMove.match(netstring)) {
+
+    if ((CurrentPosition.sidehint && EngineWhite) ||
+	(!CurrentPosition.sidehint && !EngineWhite) ) {
+      // engine sent illegal move string during HIS move
+      // happens when FEN string is sent to GNU Chess 5 and
+      // it doesn't understand the castling permissions
+      global.output->append("engine claimed illegal move but we didn't move, ignoring it.", global.Colors.TextBright, IM_IGNORE);
+      return;
+    }
+
     CurrentPosition=LegalityBackup;
     refgame=getGame();
     backMove();
@@ -173,19 +196,19 @@ void XBoardProtocolParser::receiveString(char *netstring) {
 }
 
 
-void XBoardProtocolParser::advanceMove() {
+void XBoardProtocol::advanceMove() {
   if (MoveBlack)
     MoveNumber++;
   MoveBlack=!MoveBlack;
 }
 
-void XBoardProtocolParser::backMove() {
+void XBoardProtocol::backMove() {
   if (!MoveBlack)
     MoveNumber--;
   MoveBlack=!MoveBlack;
 }
 
-void XBoardProtocolParser::sendMove(int x1,int y1,int x2,int y2,int prom) {
+void XBoardProtocol::sendMove(int x1,int y1,int x2,int y2,int prom) {
   ChessGame *refgame;
   char move[6];
   char mymove[12],ppfmm[18]; // [p]retty [p]rint [f]or [m]y [m]ove
@@ -232,7 +255,7 @@ void XBoardProtocolParser::sendMove(int x1,int y1,int x2,int y2,int prom) {
   global.BoardList.front()->setCanMove(false);
 }
 
-void XBoardProtocolParser::finalize() {
+void XBoardProtocol::finalize() {
   ChessGame *refgame;
   refgame=getGame();
   if (refgame) {
@@ -243,7 +266,7 @@ void XBoardProtocolParser::finalize() {
     global.network->close();
 }
 
-void XBoardProtocolParser::gameOver(ExtPatternMatcher &pm,GameResult gr,
+void XBoardProtocol::gameOver(ExtPatternMatcher &pm,GameResult gr,
                                     int hasreason=1)
 {
   char reason[64];
@@ -269,26 +292,26 @@ void XBoardProtocolParser::gameOver(ExtPatternMatcher &pm,GameResult gr,
   global.network->close();
 }
 
-void XBoardProtocolParser::resign() {
+void XBoardProtocol::resign() {
   global.network->writeLine("resign");
   global.status->setText("Resigned.");
 }
 
-void XBoardProtocolParser::draw() {
+void XBoardProtocol::draw() {
   global.network->writeLine("draw");
   global.status->setText("<XBoard Protocol> draw request sent");
 }
 
-void XBoardProtocolParser::adjourn() {
+void XBoardProtocol::adjourn() {
   global.status->setText("<XBoard Protocol> Adjourning not supported");
 }
 
-void XBoardProtocolParser::abort() {
+void XBoardProtocol::abort() {
   finalize();
   global.status->setText("<XBoard Protocol> Session Aborted");
 }
 
-ChessGame * XBoardProtocolParser::getGame() {
+ChessGame * XBoardProtocol::getGame() {
   ChessGame *refgame;
   refgame=global.getGame(XBOARD_GAME);
   if (!refgame) {
@@ -298,7 +321,7 @@ ChessGame * XBoardProtocolParser::getGame() {
   return refgame;
 }
 
-int XBoardProtocolParser::run() {
+int XBoardProtocol::run() {
   createDialog();
   if (!runDialog()) {
     destroyDialog();
@@ -318,14 +341,24 @@ int XBoardProtocolParser::run() {
   return(loadEngine());
 }
 
-int XBoardProtocolParser::run(EngineBookmark *bm) {  
+int XBoardProtocol::run(EngineBookmark *bm) {  
   loadBookmark(bm);
   return(loadEngine());
 }
 
-int XBoardProtocolParser::loadEngine() {
+gboolean xb_initengine_timeout(gpointer data) {
+  XBoardProtocol *me;
+  me=(XBoardProtocol *) data;
+
+  global.status->setText("Initializing engine");
+
+  me->initEngine();
+  return FALSE;
+}
+
+int XBoardProtocol::loadEngine() {
   PipeConnection *link;
-  global.debug("XBoardProtocolParser","loadEngine",EngineCommandLine);
+  global.debug("XBoardProtocol","loadEngine",EngineCommandLine);
 
   if (strlen(EngineRunDir))
     sprintf(FullCommand,"cd %s ; %s",EngineRunDir,EngineCommandLine);
@@ -334,10 +367,17 @@ int XBoardProtocolParser::loadEngine() {
 
   link=new PipeConnection("/bin/sh","-c",FullCommand,0,0);
 
+  // only gnu chess 4 has trouble with it
+  if (need_handshake)
+    link->setHandshake("xboard\nprotover 2\n");
+
   if (link->open()==0) { 
-    global.status->setText("Engine started");
+    global.status->setText("Engine started (2 sec timeout)");
     global.network=link;
-    initEngine();
+
+    // run initEngine after 2 seconds
+    timeout_id=gtk_timeout_add(2000,xb_initengine_timeout,(gpointer) this);
+
     if (global.network->isConnected())
       return 1;
     else {
@@ -351,11 +391,10 @@ int XBoardProtocolParser::loadEngine() {
   return 0;
 }
 
-void XBoardProtocolParser::initEngine() {
-  char z[64];
+void XBoardProtocol::initEngine() {
+  char z[256];
   createGame();
-  global.network->writeLine("xboard");
-  global.network->writeLine("protover 2");
+  // xboard/protover already sent in open() call
   global.network->writeLine("nopost");
 
   if (ThinkAlways)
@@ -375,16 +414,100 @@ void XBoardProtocolParser::initEngine() {
     global.network->writeLine(z);
   }
 
-  if (EngineWhite) {
-    global.network->writeLine("white");
+  lateInit();
+  endInit();
+}
+
+void XBoardProtocol::endInit() {
+  char z[256];
+  sprintf(z,"%s engine started.", ComputerName);
+  global.status->setText(z);
+}
+
+void XBoardProtocol::lateInit() {
+  if (got_init_pos) {    
+
+    global.network->writeLine("force");
+
+    setupPosition();
+
+    // the a2a3 thing suggested in the protocol spec 
+    // makes GNU chess 4 go nuts
+
+    if (initpos.sidehint)
+      global.network->writeLine("white");
+    else
+      global.network->writeLine("black");
+
     global.network->writeLine("go");
+    
+  } else {
+    
+    if (EngineWhite) {
+      global.network->writeLine("white");
+      global.network->writeLine("go");
+    }
+    
   }
 }
 
-void XBoardProtocolParser::createGame() {
+void XBoardProtocol::setupPosition() {
+  char z[256];
+
+  if (!got_init_pos) return;
+
+  if (supports_setboard) {
+    string s;
+
+    s="setboard ";
+    s+=initpos.getFEN();
+    strcpy(z,s.c_str());
+    global.network->writeLine(z);    
+
+  } else {
+    char assoc[128];
+    int i,j;
+    piece p;
+
+    global.network->writeLine("edit");
+    global.network->writeLine("#");
+
+    assoc[PAWN]  ='P';
+    assoc[KNIGHT]='N';
+    assoc[BISHOP]='B';
+    assoc[ROOK]  ='R';
+    assoc[QUEEN] ='Q';
+    assoc[KING]  ='K';
+
+    for(i=0;i<8;i++)
+      for(j=0;j<8;j++) {
+	p=initpos.getPiece(i,j);
+	if ((p&COLOR_MASK) == WHITE) {
+	  sprintf(z,"%c%c%d",assoc[p&PIECE_MASK],'a'+i,1+j);
+	  global.network->writeLine(z);
+	}
+      }
+    
+    global.network->writeLine("c");
+
+    for(i=0;i<8;i++)
+      for(j=0;j<8;j++) {
+	p=initpos.getPiece(i,j);
+	if ((p&COLOR_MASK) == BLACK) {
+	  sprintf(z,"%c%c%d",assoc[p&PIECE_MASK],'a'+i,1+j);
+	  global.network->writeLine(z);
+	}
+      }
+
+    global.network->writeLine(".");
+  }
+
+}
+
+void XBoardProtocol::createGame() {
   ChessGame *cg;
   Position *pos;
-  global.debug("XBoardProtocolParser","createGame");
+  global.debug("XBoardProtocol","createGame");
 
   MoveNumber=1;
   MoveBlack=0;
@@ -415,11 +538,19 @@ void XBoardProtocolParser::createGame() {
   cg->fireWhiteClock(0,0);
 
   pos=new Position();
-  cg->updatePosition(*pos,1,0,0,0,"0. startpos");
+
+  if (got_init_pos) {
+    (*pos)=initpos;
+    CurrentPosition = initpos;
+    LegalityBackup  = initpos;
+    MoveBlack = pos->sidehint ? 0 : 1;
+  }
+
+  cg->updatePosition(*pos,1,pos->sidehint ? 0:1,0,0,"0. startpos");
   delete pos;
 }
 
-void XBoardProtocolParser::dumpGame() {
+void XBoardProtocol::dumpGame() {
   ChessGame *cg;
 
   cg=global.getGame(XBOARD_GAME);
@@ -430,10 +561,12 @@ void XBoardProtocolParser::dumpGame() {
 
 }
 
-void XBoardProtocolParser::parseFeatures() {
+void XBoardProtocol::parseFeatures() {
   ExtPatternMatcher sf,nf;
   ChessGame *cg;
   char *cand,*p;
+  int n;
+  bool gotdone=false;
 
   sf.set("%b%s=\"*\"*");
   nf.set("%b%s=%n*");
@@ -456,24 +589,55 @@ void XBoardProtocolParser::parseFeatures() {
       continue;
     }
     if (nf.match(cand)) {
+      p=nf.getSToken(0);
+
+      if (!strcmp(p,"setboard")) {
+	n=atoi(nf.getNToken(0));
+	if (n) supports_setboard=true;
+      }
+
+      if (!strcmp(p,"done")) {
+	n=atoi(nf.getNToken(0));
+	if (n)
+	  gotdone=true;
+	else {
+	  // engine asked more time
+	  global.status->setText("Engine asked more time to startup, waiting.");
+	  if (timeout_id >= 0)
+	    gtk_timeout_remove(timeout_id);
+	  
+	  // 1 hour, as per Mann's doc
+	  timeout_id=gtk_timeout_add(3600000, xb_initengine_timeout,
+				     (gpointer) this);
+	}
+	
+      }
+
       cand=nf.getStarToken(0);
       continue;
     }
     break;
   }
+
+  if (gotdone && timeout_id >= 0) {
+    gtk_timeout_remove(timeout_id);
+    timeout_id=-1;
+    global.status->setText("Engine loaded.");
+    initEngine();
+  }
 }
 
-char * XBoardProtocolParser::getDialogName() {
+char * XBoardProtocol::getDialogName() {
   return("Play against Engine");
 }
 
-char * XBoardProtocolParser::getComputerName() {
+char * XBoardProtocol::getComputerName() {
   return(ComputerName);
 }
 
 // ----- the bloated world of GUI -------
 
-void XBoardProtocolParser::createDialog() {
+void XBoardProtocol::createDialog() {
   GtkWidget *v,*v2;
   GtkWidget *cfr,*rd[2],*ok,*cancel,*bhb,*hs;
   GtkWidget *f2,*tabl,*l2,*spm,*tl1,*tl2,*te1,*tv;
@@ -636,9 +800,19 @@ void XBoardProtocolParser::createDialog() {
   engctl_ewhite=rd[1];
   engctl_engcmd=path3;
   engctl_engdir=path4;
+
+  if (got_init_pos) {
+    if (initpos.sidehint) // white to move
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rd[1]), TRUE);
+    else
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rd[0]), TRUE);
+
+    gtk_widget_set_sensitive(rd[0],FALSE);
+    gtk_widget_set_sensitive(rd[1],FALSE);
+  }
 }
 
-void XBoardProtocolParser::readDialog() {
+void XBoardProtocol::readDialog() {
 
   EngineWhite=0;
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(engctl_ewhite)))
@@ -670,7 +844,7 @@ void XBoardProtocolParser::readDialog() {
   }
 }
 
-void XBoardProtocolParser::makeBookmarkCaption() {
+void XBoardProtocol::makeBookmarkCaption() {
   tstring t,u;
   string x, *p;
   char caption[1024];
@@ -680,10 +854,14 @@ void XBoardProtocolParser::makeBookmarkCaption() {
   t.set(EngineCommandLine);
   x=EngineCommandLine;
   if (x.empty()) x="<blank?>";
-  while ( ( p=t.token("/") ) != 0 ) {
+  
+  p=t.token(" ");
+  if (p != 0) {
     u.set(*p);
-    x=*(u.token(" "));
+    while( (p=u.token("/")) != 0 )
+      x=*p;
   }
+  
   sprintf(caption,"Play %s as %s vs. %s (%d secs/move, maxdepth %d, think always: %s)",
 	  ChessGame::variantName(ebm->mode),
 	  ebm->humanwhite?"White":"Black",
@@ -694,7 +872,7 @@ void XBoardProtocolParser::makeBookmarkCaption() {
   ebm->caption = caption;
 }
 
-void XBoardProtocolParser::loadBookmark(EngineBookmark *bm) {
+void XBoardProtocol::loadBookmark(EngineBookmark *bm) {
   SecsPerMove = bm->secspermove;
   MaxDepth    = bm->maxply;
   ThinkAlways = (bm->think!=0);
@@ -702,9 +880,12 @@ void XBoardProtocolParser::loadBookmark(EngineBookmark *bm) {
   Variant     = bm->mode;
   strcpy(EngineRunDir, bm->directory.c_str() );
   strcpy(EngineCommandLine, bm->cmdline.c_str() );
+
+  if (got_init_pos)
+    EngineWhite=initpos.sidehint ? 1 : 0;
 }
 
-int XBoardProtocolParser::runDialog() {
+int XBoardProtocol::runDialog() {
   GotResult=0;
   gtk_widget_show(eng_dlg);
   gtk_grab_add(eng_dlg);
@@ -714,19 +895,19 @@ int XBoardProtocolParser::runDialog() {
   return(GotResult==1);
 }
 
-void XBoardProtocolParser::destroyDialog() {
+void XBoardProtocol::destroyDialog() {
   gtk_widget_destroy(eng_dlg);
 }
 
 void xboard_eng_ok(GtkWidget *w,gpointer data) {
-  XBoardProtocolParser *me;
-  me=(XBoardProtocolParser *)data;
+  XBoardProtocol *me;
+  me=(XBoardProtocol *)data;
   me->GotResult=1;
 }
 
 void xboard_eng_cancel(GtkWidget *w,gpointer data) {
-  XBoardProtocolParser *me;
-  me=(XBoardProtocolParser *)data;
+  XBoardProtocol *me;
+  me=(XBoardProtocol *)data;
   me->GotResult=-1;
 }
 
