@@ -1,4 +1,4 @@
-/* $Id: proto_xboard.h,v 1.8 2002/04/24 05:25:09 bergo Exp $ */
+/* $Id: proto_xboard.h,v 1.9 2002/06/14 17:32:57 bergo Exp $ */
 
 /*
 
@@ -37,10 +37,10 @@ class EngineBookmark;
 // a generic xboard protocol (version 2) parser
 // http://www.tim-mann.org/xboard/engine-intf.html
 
-class XBoardProtocolParser : public ProtocolParser {
+class XBoardProtocol : public EngineProtocol {
 
  public:
-  XBoardProtocolParser();
+  XBoardProtocol();
 
   virtual void receiveString(char *netstring);
   virtual void sendMove(int x1,int y1,int x2,int y2,int prom);
@@ -57,7 +57,16 @@ class XBoardProtocolParser : public ProtocolParser {
   virtual void initEngine();
   virtual void readDialog();
 
+  virtual void setInitialPosition(Position *p);
+
  protected:
+  // initEngine work after variant settings and "new" command,
+  // common to most classes
+  virtual void lateInit();
+
+  // set status bar after all is done
+  virtual void endInit();
+  
   virtual void loadBookmark(EngineBookmark *bm);
   virtual void createDialog();
   virtual void createGame();
@@ -74,6 +83,8 @@ class XBoardProtocolParser : public ProtocolParser {
   void gameOver(ExtPatternMatcher &pm,GameResult gr,int hasreason=1);
   ChessGame *getGame();
 
+  virtual void setupPosition();
+
   GtkWidget *eng_dlg, *eng_book, *engctl_engcmd, *engctl_engdir;
 
   int  EngineWhite;
@@ -87,7 +98,11 @@ class XBoardProtocolParser : public ProtocolParser {
   char EngineRunDir[256];
   char ComputerName[64];  
 
-  int want_path_pane;
+  bool supports_setboard;
+
+  int  want_path_pane;
+  bool need_handshake;
+  int  timeout_id;
 
   ExtPatternMatcher WhiteWin[2];
   ExtPatternMatcher BlackWin[2];
@@ -104,6 +119,9 @@ class XBoardProtocolParser : public ProtocolParser {
 
   EngineBookmark *ebm;
 
+  Position initpos;
+  bool     got_init_pos;
+
  private:
   int runDialog();
   void destroyDialog();
