@@ -1,4 +1,4 @@
-/* $Id: protocol.cc,v 1.21 2002/02/13 03:53:56 bergo Exp $ */
+/* $Id: protocol.cc,v 1.22 2002/06/14 17:32:57 bergo Exp $ */
 
 /*
 
@@ -28,87 +28,87 @@
 #include "protocol.h"
 #include "global.h"
 
-void ProtocolParser::finalize() {
+void Protocol::finalize() {
 
 }
 
-int  ProtocolParser::hasAuthenticationPrompts() {
+int  Protocol::hasAuthenticationPrompts() {
   return 0;
 }
 
-void ProtocolParser::discardGame(int gameid) {
+void Protocol::discardGame(int gameid) {
 
 }
 
-void ProtocolParser::queryGameList(GameListConsumer *glc) {
+void Protocol::queryGameList(GameListConsumer *glc) {
   glc->endOfList();
 }
 
-void ProtocolParser::queryAdList(GameListConsumer *glc) {
+void Protocol::queryAdList(GameListConsumer *glc) {
   glc->endOfList();
 }
 
-void ProtocolParser::observe(int gameid) {
+void Protocol::observe(int gameid) {
 
 }
 
-void ProtocolParser::answerAd(int adid) {
+void Protocol::answerAd(int adid) {
 
 }
 
-void ProtocolParser::exaForward(int n) {
+void Protocol::exaForward(int n) {
 
 }
 
-void ProtocolParser::exaBackward(int n) {
+void Protocol::exaBackward(int n) {
 
 }
 
-void ProtocolParser::sendDrop(piece p,int x,int y) {
+void Protocol::sendDrop(piece p,int x,int y) {
   // not all protocols need to support {bug,crazy}house
 }
 
-void ProtocolParser::refreshSeeks() {
+void Protocol::refreshSeeks() {
   // not all protocols need to manage a seek table
 }
 
-vector<string *> * ProtocolParser::getPlayerActions() {
+vector<string *> * Protocol::getPlayerActions() {
   return NULL;
 }
 
-vector<string *> * ProtocolParser::getGameActions() {
+vector<string *> * Protocol::getGameActions() {
   return NULL;
 }
 
-void ProtocolParser::callPlayerAction(char *player, string *action) {
+void Protocol::callPlayerAction(char *player, string *action) {
 
 }
 
-void ProtocolParser::callGameAction(int gameid, string *action) {
+void Protocol::callGameAction(int gameid, string *action) {
 
 }
 
-void NullProtocolParser::receiveString(char *netstring) {
+void NullProtocol::receiveString(char *netstring) {
   global.output->append(netstring,0xffffff);
 }
 
-void NullProtocolParser::sendMove(int x1,int y1,int x2,int y2,int prom) {
+void NullProtocol::sendMove(int x1,int y1,int x2,int y2,int prom) {
 
 }
 
-void NullProtocolParser::resign() {
+void NullProtocol::resign() {
 
 }
 
-void NullProtocolParser::draw() {
+void NullProtocol::draw() {
 
 }
 
-void NullProtocolParser::adjourn() {
+void NullProtocol::adjourn() {
 
 }
 
-void NullProtocolParser::abort() {
+void NullProtocol::abort() {
 
 }
 
