@@ -1,4 +1,4 @@
-/* $Id: protocol.h,v 1.20 2001/10/13 00:22:10 bergo Exp $ */
+/* $Id: protocol.h,v 1.21 2002/06/14 17:32:57 bergo Exp $ */
 
 /*
 
@@ -36,7 +36,7 @@
 #include "tstring.h"
 #include "position.h"
 
-class ProtocolParser {
+class Protocol {
  public:
   virtual void receiveString(char *netstring)=0;
   virtual void sendMove(int x1,int y1,int x2,int y2,int prom)=0;
@@ -68,7 +68,7 @@ class ProtocolParser {
   virtual void callGameAction(int gameid, string *action);
 };
 
-class NullProtocolParser : public ProtocolParser {
+class NullProtocol : public Protocol {
  public:
   void receiveString(char *netstring);
   void sendMove(int x1,int y1,int x2,int y2,int prom);
@@ -79,6 +79,24 @@ class NullProtocolParser : public ProtocolParser {
   virtual void abort();
 };
 
+class EngineBookmark;
+
+class EngineProtocol : public Protocol {
+ public:
+
+  // both run methods should return 0 in case of failure, anything
+  // else in case of success
+
+  // should open a dialog asking parameters for the engine
+  virtual int run()=0;
+
+  // should run the engine with parameters from the bookmark
+  virtual int run(EngineBookmark *bm)=0;
+
+  virtual void setInitialPosition(Position *p)=0;
+
+};
+
 // keeping each protocol in a separate source file,
 // to minimize cvs conflicts when (hopefully)
 // more people apply to write protocols
