@@ -1,4 +1,4 @@
-/* $Id: script.cc,v 1.8 2002/04/24 01:16:33 bergo Exp $ */
+/* $Id: script.cc,v 1.9 2002/06/26 01:07:00 bergo Exp $ */
 
 /*
 
@@ -296,6 +296,8 @@ void scripti_kill(GtkWidget *w,gpointer data) {
   global.removeAgent(me->child);
   me->child->close();
   gtk_container_remove(GTK_CONTAINER(global.killbox), me->widget);
+  if (global.toplevelwidget)
+    gtk_widget_queue_resize(global.toplevelwidget);
   //gtk_widget_destroy(me->widget);
   delete me->child;
   delete me;
