@@ -1,4 +1,4 @@
-/* $Id: sound.cc,v 1.10 2002/04/24 01:16:33 bergo Exp $ */
+/* $Id: sound.cc,v 1.12 2002/06/17 03:24:31 bergo Exp $ */
 
 /*
 
@@ -40,11 +40,33 @@
 #include "global.h"
 #include "tstring.h"
 #include "util.h"
-#include "config.h"
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#define SOME_DRIVER 1
 
 #ifdef HAVE_SYS_SOUNDCARD_H
+
+#define OSS_DRIVER 1
 #include <sys/soundcard.h>
+
+#elif defined HAVE_SYS_AUDIOIO_H
+
+#define OPENBSD_DRIVER 1
+#include <sys/audioio.h>
+
+// Solaris has sys/audio.h but is incompatible with OpenBSD
+#ifndef AUMODE_PLAY
+#undef SOME_DRIVER
+#undef OPENBSD_DRIVER
+#endif
+
+#else
+
+#undef SOME_DRIVER
+
 #endif
 
 SoundEvent::SoundEvent() {
@@ -171,16 +193,25 @@ void SoundEvent::play() {
 }
 
 void SoundEvent::sine_beep(char *device,int pitch,int duration) {
-#ifdef HAVE_SYS_SOUNDCARD_H
-  int rate=11025;         // Hz
-  int format=AFMT_U8;
-  int channels=1;
+#ifdef SOME_DRIVER
+  int rate=11025; // Hz
   int interval;
 
   unsigned char *wave;
   int bl,fd,i,ts;
   double r,s;
+#endif // SOME
 
+#ifdef OSS_DRIVER
+  int format=AFMT_U8;
+  int channels=1;
+#endif
+
+#ifdef OPENBSD_DRIVER
+  audio_info_t ai;
+#endif // OPENBSD
+
+#ifdef SOME_DRIVER
   interval=120*rate/1000; // 120 msec
 
   wave=(unsigned char *)malloc(ts = (Count*(bl=(rate*duration)/1000) + (Count-1)*interval) );
@@ -205,7 +236,9 @@ void SoundEvent::sine_beep(char *device,int pitch,int duration) {
   fd=open(device,O_WRONLY);
   if (fd<0)
     goto leave2;
-  
+#endif // SOME
+
+#ifdef OSS_DRIVER
   if (ioctl(fd,SNDCTL_DSP_SETFMT,&format)==-1)
     goto leave1;
 
@@ -214,17 +247,39 @@ void SoundEvent::sine_beep(char *device,int pitch,int duration) {
 
   if (ioctl(fd,SNDCTL_DSP_SPEED,&rate)==-1)
     goto leave1;
+#endif // OSS
 
+#ifdef OPENBSD_DRIVER
+  AUDIO_INITINFO(&ai);
+  ai.mode             = AUMODE_PLAY;
+  ai.play.sample_rate = rate;
+  ai.play.channels    = 1;
+  ai.play.encoding    = AUDIO_ENCODING_ULINEAR;
+
+  if (ioctl(fd,AUDIO_SETINFO,&ai)==-1)
+    goto leave1;
+#endif // OPENBSD
+
+#ifdef SOME_DRIVER
   for(i=0;i<ts;)
     i+=::write(fd,&wave[i],ts-i);
+#endif // SOME
 
+#ifdef OSS_DRIVER
   ioctl(fd,SNDCTL_DSP_POST,0);
+#endif // OSS
 
+#ifdef OPENBSD_DRIVER
+  ioctl(fd,AUDIO_DRAIN,0);
+#endif // OPENBSD
+
+#ifdef SOME_DRIVER
  leave1:
   close(fd);
  leave2:
   free(wave);
-#endif
+#endif // SOME
+
 }
 
 char *SoundEvent::getDescription() {
