@@ -1,10 +1,10 @@
-/* $Id: sound.h,v 1.6 2001/09/30 17:48:23 bergo Exp $ */
+/* $Id: sound.h,v 1.8 2002/04/24 01:16:33 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -52,8 +52,7 @@ class SoundEvent {
   int operator==(SoundEvent &se);
   int operator!=(SoundEvent &se);
 
-  void read(TString &rcline);
-  void write(FILE *f,const char *key, int n);
+  void read(tstring &rcline);
 
   void safePlay();
   void play();
@@ -74,6 +73,8 @@ class SoundEvent {
   char pvt[128];
 };
 
+ostream & operator<<(ostream &s,  SoundEvent e);
+
 class SoundEventDialog : public ModalDialog {
  public:
   SoundEventDialog(SoundEvent *src, SoundEventChangeListener *listener);
