@@ -14,15 +14,15 @@
 #include <iostream>
 #include <fstream>
 
-#ifdef NEED_STL_HACK
 using std::list;
 using std::vector;
 using std::deque;
 using std::stack;
 using std::string;
 
+using std::ostream;
+using std::istream;
 using std::ofstream;
 using std::ifstream;
-#endif
 
 #endif
