@@ -1,4 +1,4 @@
-/* $Id: text.cc,v 1.19 2002/02/13 03:53:56 bergo Exp $ */
+/* $Id: text.cc,v 1.22 2002/04/28 21:04:34 bergo Exp $ */
 
 /*
 
@@ -34,6 +34,7 @@
 #include "widgetproxy.h"
 #include "global.h"
 #include "tstring.h"
+#include "stl.h"
 
 #include "icon-console.xpm"
 #include "addcons.xpm"
@@ -46,7 +47,7 @@ OutputPane::~OutputPane() {
 
 }
 
-Text::Text() : Searchable() {
+Text::Text() {
   GdkFont *fxd;
   GtkWidget *hb;
 
@@ -107,7 +108,7 @@ void Text::findTextNext() {
   char z[256];
   bool wrapped = false;
 
-  if (SearchString[0]==0) {
+  if (SearchString.empty()) {
     global.status->setText("No previous search.");
     return;
   }
@@ -115,7 +116,8 @@ void Text::findTextNext() {
   if (LastMatch == 0) wrapped = true;
   if (LastMatch < 0) LastMatch = 0;
 
-  LastMatch = gtk_ftext_find_text_upwards(GTK_FTEXT(text), SearchString,
+  LastMatch = gtk_ftext_find_text_upwards(GTK_FTEXT(text), 
+					  SearchString.c_str(),
 					  0, LastMatch - 1 , TRUE);
 
   if (LastMatch < 0) {
@@ -413,6 +415,7 @@ void DetachedConsole::show() {
 void DetachedConsole::injectInput() {
   global.input->userInput(gtk_entry_get_text(GTK_ENTRY(inputbox)));
   gtk_entry_set_text(GTK_ENTRY(inputbox),"\0");
+  hcursor=global.inputhistory->getCursor();
 }
 
 void DetachedConsole::historyUp() {
@@ -610,10 +613,11 @@ TextFilter::~TextFilter() {
 }
   
 void TextFilter::set(char *t) {
-  TString T(1024);
+  tstring T;
   ExtPatternMatcher *epm;
   char local[1024],single[512];
   char *p,*q;
+  string *P;
 
   cleanUp();
   if (!strlen(t))
@@ -630,14 +634,14 @@ void TextFilter::set(char *t) {
 
   T.set(local);
 
-  while( ( p=T.token("|\n\r") ) != 0 ) {
-    if (strlen(p)) {
+  while( ( P=T.token("|\n\r") ) != 0 ) {
+    if (! P->empty() ) {
       epm=new ExtPatternMatcher();
       single[0]=0;
-      if (p[0]!='*')
+      if (P->at(0) != '*')
 	strcat(single,"*");
-      strcat(single,p);
-      if (p[strlen(p)-1]!='*')
+      strcat(single,P->c_str());
+      if (P->at(P->length()-1) != '*')
 	strcat(single,"*");
       epm->set(single);
       thefilter.push_back(epm);
@@ -680,10 +684,6 @@ void TextFilter::cleanUp() {
 
 // SearchTextDialog and its interface
 
-Searchable::Searchable() {
-  SearchString[0]=0;
-}
-
 SearchTextDialog::SearchTextDialog(Searchable *src) 
   : ModalDialog("Search Text")
 {
@@ -728,7 +728,7 @@ SearchTextDialog::SearchTextDialog(Searchable *src)
   gtk_widget_show(h);
   gtk_widget_show(v);
 
-  gtk_entry_set_text(GTK_ENTRY(text), searchable->SearchString);
+  gtk_entry_set_text(GTK_ENTRY(text), searchable->SearchString.c_str());
 
   setDismiss(GTK_OBJECT(can),"clicked");
   gtk_signal_connect(GTK_OBJECT(ok),"clicked",
@@ -740,8 +740,7 @@ SearchTextDialog::SearchTextDialog(Searchable *src)
 void std_ok(GtkWidget *w, gpointer data) {
   SearchTextDialog *me;
   me=(SearchTextDialog *)data;
-  strncpy(me->searchable->SearchString,
-	  gtk_entry_get_text(GTK_ENTRY(me->text)), 128);
+  me->searchable->SearchString=gtk_entry_get_text(GTK_ENTRY(me->text));
   me->searchable->execSearch();
   me->release();
 }
