@@ -1,4 +1,4 @@
-/* $Id: util.cc,v 1.18 2002/04/24 01:16:33 bergo Exp $ */
+/* $Id: util.cc,v 1.22 2002/06/28 01:33:05 bergo Exp $ */
 
 /*
 
@@ -29,6 +29,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
+#include <ctype.h>
 #include "global.h"
 #include "prefix.h"
 #include "stl.h"
@@ -93,7 +94,11 @@ int FileFinder::find(string &name, string &out) {
     return 0;
   }
   f.close();
-  out=name;
+
+  // avoid problems with scripts when . is not in the PATH
+  out="./";
+  out+=name;
+
   return 1;  
 }
 
@@ -118,7 +123,8 @@ int FileFinder::find(char *name,char *fullpath) {
     return 0;
   }
   f.close();
-  strcpy(fullpath,name);
+  strcpy(fullpath,"./");
+  strcat(fullpath,name);
   return 1;
 }
 
@@ -171,6 +177,20 @@ int ExactStringPat::tryMatch(list<char>::iterator & first,
   return 0;
 }
 
+int CIExactStringPat::tryMatch(list<char>::iterator & first,
+			       list<char>::iterator & last) {
+  list<char>::iterator mine,their;
+  
+  for(mine=pattern.begin(), their=first;
+      (mine!=pattern.end()) && (their!=last);
+      mine++, their++) {
+    if (toupper(*mine) != toupper(*their)) return -1;
+  }
+  if (mine!=pattern.end()) return -1;
+  last=their;
+  return 0;
+}
+
 KleeneStarPat::KleeneStarPat() {
   reset();
 }
