@@ -1,4 +1,4 @@
-/* $Id: util.h,v 1.11 2002/04/24 01:16:33 bergo Exp $ */
+/* $Id: util.h,v 1.12 2002/06/26 01:07:00 bergo Exp $ */
 
 /*
 
@@ -75,10 +75,18 @@ class Pattern {
 class ExactStringPat : public Pattern {
  public:
   ExactStringPat(char *pat);
+  virtual int tryMatch(list<char>::iterator & first,
+		       list<char>::iterator & last);
+ protected:
+  list<char> pattern;
+};
+
+// same thing, case insensitive
+class CIExactStringPat : public ExactStringPat {
+ public:
+  CIExactStringPat(char *pat) : ExactStringPat(pat) { }
   int tryMatch(list<char>::iterator & first,
 	       list<char>::iterator & last);
- private:
-  list<char> pattern;
 };
 
 // you probably know this simply as the '*' wildcard
