@@ -1,4 +1,4 @@
-/* $Id: widgetproxy.cc,v 1.18 2002/04/24 01:16:33 bergo Exp $ */
+/* $Id: widgetproxy.cc,v 1.19 2002/06/14 17:32:57 bergo Exp $ */
 
 /*
 
@@ -31,6 +31,9 @@
 #include "widgetproxy.h"
 #include "global.h"
 
+// ColorSpace class
+#include "pieces.h"
+
 WidgetProxy::WidgetProxy() {
   widget=0;
 }
@@ -515,6 +518,10 @@ void LayoutBox::drawLine(int x,int y,int w,int h) {
   gdk_draw_line(Target,GC,x,y,w,h);
 }
 
+void LayoutBox::drawEllipse(int x,int y,int w,int h,bool fill) {
+  gdk_draw_arc(Target,GC,fill?TRUE:FALSE,x,y,w,h,0,360*64);
+}
+
 void LayoutBox::consumeTop(int v) { Y+=v; H-=v; }
 void LayoutBox::consumeBottom(int v) { H-=v; }
 
@@ -595,3 +602,41 @@ bool LayoutBox::drawBoxedString(int x,int y,int w,int h,
   return true;
 
 } // method
+
+int LayoutBox::drawButton(int x, int y, int w, int h, char *s, int font,
+			  int fg,int bg)
+{
+  ColorSpace cs;
+  int i,j;
+
+  if (s) {
+    j=stringWidth(font,s);
+    i=fontHeight(font);
+  } else {
+    j=16;
+    i=10;
+  }
+
+  if (w<0) w=j+10;
+
+  setColor(bg);
+  drawRect(x,y,w,h,true);
+
+  setColor(cs.darker(bg));
+  drawRect(x+1,y+1,w-2,h-2,false);
+  drawRect(x+2,y+2,w-4,h-4,false);
+
+  setColor(cs.lighter(bg));
+  drawLine(x+1,y+1,x+w-1,y+1);
+  drawLine(x+2,y+2,x+w-2,y+2);
+  drawLine(x+1,y+1,x+1,y+h-1);
+  drawLine(x+2,y+2,x+2,y+h-2);
+
+  setColor(fg);
+  drawRect(x,y,w,h,false);
+
+  if (s)
+    drawString(x+(w-j)/2, y + h - ((3+h-i)/2) - 1,font,s);
+
+  return w;
+}
