@@ -1,4 +1,4 @@
-/* $Id: widgetproxy.h,v 1.16 2002/04/23 07:55:09 bergo Exp $ */
+/* $Id: widgetproxy.h,v 1.17 2002/06/14 17:32:57 bergo Exp $ */
 
 /*
 
@@ -158,9 +158,13 @@ class LayoutBox {
   void setColor(int c);
   void drawRect(int x,int y,int w,int h,bool fill);
   void drawLine(int x,int y,int w,int h);
+  void drawEllipse(int x,int y,int w,int h,bool fill);
   void drawString(int x,int y, int f, char *s);
   void drawString(int x,int y, int f, string &s);
   void drawSubstring(int x,int y, int f, char *s,int len);
+  int  drawButton(int x,int y,int w,int h, char *s,int font,
+		  int fg,int bg);
+
   int  stringWidth(int f,char *s);
   int  substringWidth(int f,char *s,int len);
   int  fontHeight(int f);
